@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -6,6 +6,7 @@
 #include <string>
 #include <string.h>
 #include <stdlib.h>
+#include <iostream>
 #include <fstream>
 #include "GmshConfig.h"
 #include "GmshDefines.h"
@@ -28,6 +29,18 @@
 #endif
 #endif
 
+#if defined(HAVE_PETSC)
+#include "petsc.h"
+#endif
+
+#if defined(HAVE_OCC)
+#include "Standard_Version.hxx"
+#endif
+
+#if defined(HAVE_MED)
+#include "med.h"
+#endif
+
 #if defined(HAVE_POST)
 #include "PView.h"
 #endif
@@ -36,6 +49,10 @@
 #include "periodical.h"
 #endif
 
+#if defined(HAVE_PARSER)
+#include "Parser.h"
+#endif
+
 int GetGmshMajorVersion(){ return GMSH_MAJOR_VERSION; }
 int GetGmshMinorVersion(){ return GMSH_MINOR_VERSION; }
 int GetGmshPatchVersion(){ return GMSH_PATCH_VERSION; }
@@ -53,7 +70,7 @@ std::vector<std::pair<std::string, std::string> > GetUsage()
   typedef std::pair<std::string, std::string> mp;
   std::vector<mp> s;
   s.push_back(mp("Geometry options:", ""));
-  s.push_back(mp("-0",                 "Output unrolled geometry, then exit"));
+  s.push_back(mp("-0",                 "Output model, then exit"));
   s.push_back(mp("-tol float",         "Set geometrical tolerance"));
   s.push_back(mp("-match",             "Match geometries and meshes"));
   s.push_back(mp("Mesh options:", ""));
@@ -70,7 +87,7 @@ std::vector<std::pair<std::string, std::string> > GetUsage()
   s.push_back(mp("-saveall",           "Save all elements (discard physical group definitions)"));
   s.push_back(mp("-parametric",        "Save vertices with their parametric coordinates"));
   s.push_back(mp("-algo string",       "Select mesh algorithm (meshadapt, del2d, front2d, "
-                                        "delquad, del3d, front3d, mmg3d)"));
+                                        "delquad, del3d, front3d, mmg3d, pack)"));
   s.push_back(mp("-smooth int",        "Set number of mesh smoothing steps"));
   s.push_back(mp("-order int",         "Set mesh order (1, ..., 5)"));
   s.push_back(mp("-optimize[_netgen]", "Optimize quality of tetrahedral elements"));
@@ -90,13 +107,17 @@ std::vector<std::pair<std::string, std::string> > GetUsage()
   s.push_back(mp("-rand float",        "Set random perturbation factor"));
   s.push_back(mp("-bgm file",          "Load background mesh from file"));
   s.push_back(mp("-check",             "Perform various consistency checks on mesh"));
-  s.push_back(mp("-mpass int",         "Do several passes on the mesh for complex backround fields"));
+  s.push_back(mp("-mpass int",         "Do several passes on the mesh for complex background fields"));
   s.push_back(mp("-ignorePartBound",   "Ignore partitions boundaries"));
 #if defined(HAVE_FLTK)
   s.push_back(mp("Post-processing options:", ""));
   s.push_back(mp("-link int",          "Select link mode between views (0, 1, 2, 3, 4)"));
   s.push_back(mp("-combine",           "Combine views having identical names into "
                                        "multi-time-step views"));
+  s.push_back(mp("Solver options:", ""));
+  s.push_back(mp("-listen",            "Always listen to incoming connection requests"));
+  s.push_back(mp("-minterpreter string", "Name of Octave interpreter"));
+  s.push_back(mp("-pyinterpreter string", "Name of Python interpreter"));
   s.push_back(mp("Display options:", ""));
   s.push_back(mp("-n",                 "Hide all meshes and post-processing views on startup"));
   s.push_back(mp("-nodb",              "Disable double buffering"));
@@ -119,11 +140,13 @@ std::vector<std::pair<std::string, std::string> > GetUsage()
                                        "post-processing mode"));
 #endif
   s.push_back(mp("-pid",               "Print process id on stdout"));
-  s.push_back(mp("-listen",            "Always listen to incoming connection requests"));
   s.push_back(mp("-watch pattern",     "Pattern of files to merge as they become available"));
+  s.push_back(mp("-bg file",           "Load background (image or PDF) file"));
   s.push_back(mp("-v int",             "Set verbosity level"));
   s.push_back(mp("-nopopup",           "Don't popup dialog windows in scripts"));
-  s.push_back(mp("-string \"string\"", "Parse option string at startup"));
+  s.push_back(mp("-string \"string\"", "Parse command string at startup"));
+  s.push_back(mp("-setnumber name value", "Set constant number name=value"));
+  s.push_back(mp("-setstring name value", "Set constant string name=value"));
   s.push_back(mp("-option file",       "Parse option file at startup"));
   s.push_back(mp("-convert files",     "Convert files into latest binary formats, then exit"));
   s.push_back(mp("-version",           "Show version number"));
@@ -178,6 +201,7 @@ std::vector<std::pair<std::string, std::string> > GetShortcutsUsage(const std::s
   s.push_back(mp(cc + "r",         "Rename project file"));
   s.push_back(mp(cc + "s",         "Save file as"));
   s.push_back(mp("Shift+" + cc + "c", "Show clipping plane window"));
+  s.push_back(mp("Shift+" + cc + "h", "Show current options and workspace window"));
   s.push_back(mp("Shift+" + cc + "j", "Save options as default"));
   s.push_back(mp("Shift+" + cc + "m", "Show manipulator window"));
   s.push_back(mp("Shift+" + cc + "n", "Show option window"));
@@ -211,6 +235,7 @@ std::vector<std::pair<std::string, std::string> > GetShortcutsUsage(const std::s
   s.push_back(mp("Alt+Shift+l",    "Hide/show mesh lines"));
   s.push_back(mp("Alt+Shift+p",    "Hide/show mesh points"));
   s.push_back(mp("Alt+Shift+s",    "Hide/show mesh surface edges"));
+  s.push_back(mp("Alt+Shift+t",    "Same as Alt+t, but with numeric mode included"));
   s.push_back(mp("Alt+Shift+v",    "Hide/show mesh volume edges"));
   s.push_back(mp("Alt+Shift+w",    "Reverse all mesh normals"));
   s.push_back(mp("Alt+Shift+x",    "Set -X view"));
@@ -295,16 +320,7 @@ void GetOptions(int argc, char *argv[])
           i += 1;
         }
         else
-          Msg::Fatal("Missing client name and/or address of OneLab server");
-      }
-      else if(!strcmp(argv[i] + 1, "lol")) {
-        i++;
-        if(argv[i] && argv[i + 1] && argv[i + 1][0] != '-'){
-          Msg::LoadOnelabClient(argv[i], argv[i + 1]);
-          i += 2;
-        }
-        else
-          Msg::Fatal("Missing client name and/or address of OneLab server");
+          Msg::Fatal("Missing client name and/or address of ONELAB server");
       }
       else if(!strcmp(argv[i] + 1, "socket")) {
         i++;
@@ -363,7 +379,7 @@ void GetOptions(int argc, char *argv[])
         opt_mesh_partition_metis_algorithm(0, GMSH_SET, 3); // partGraphKWay w/ weights
         while (check) {
           if (argv[i]) {
-            if (!strcmp(argv[i],"triangle")) {
+            if (!strcmp(argv[i],"tri") || !strcmp(argv[i],"triangle")) {
               i++;
               opt_mesh_partition_tri_weight(0,GMSH_SET,atoi(argv[i]));
             }
@@ -387,7 +403,10 @@ void GetOptions(int argc, char *argv[])
               i++;
               opt_mesh_partition_hex_weight(0,GMSH_SET,atoi(argv[i]));
             }
-            else check = false;
+            else {
+              Msg::Error("Bad argument for 'partWeight' (%s)", argv[i]);
+              check = false;
+            }
             i++;
           }
           else check = false;
@@ -489,28 +508,11 @@ void GetOptions(int argc, char *argv[])
 #if defined(HAVE_MESH)
       else if(!strcmp(argv[i] + 1, "microstructure")) {
         i++;
-        int j;
-        int radical;
-        double max;
-        std::vector<double> properties;
-        if(argv[i]){
-          std::ifstream file(argv[i++]);
-          file >> max;
-          file >> radical;
-          properties.clear();
-          properties.resize(4*max);
-          for(j=0;j<max;j++){
-            file >> properties[4*j];
-            file >> properties[4*j+1];
-            file >> properties[4*j+2];
-            file >> properties[4*j+3];
-          }
-          voroMetal3D vm1;
-          vm1.execute(properties,radical,0.1);
-          GModel::current()->load("MicrostructurePolycrystal3D.geo");
-          voroMetal3D vm2;
-          vm2.correspondance(0.00001);
-        }
+        if(argv[i]) microstructure(argv[i++]);
+      }
+      else if(!strcmp(argv[i] + 1, "computeBestSeeds")) {
+        i++;
+        if(argv[i]) computeBestSeeds(argv[i++]);
       }
 #endif
       else if(!strcmp(argv[i] + 1, "nopopup")) {
@@ -536,6 +538,28 @@ void GetOptions(int argc, char *argv[])
         else
           Msg::Fatal("Missing string");
       }
+#if defined(HAVE_PARSER)
+      else if(!strcmp(argv[i] + 1, "setstring")) {
+        i++;
+	if (i + 1 < argc && argv[i][0] != '-' && argv[i + 1][0] != '-') {
+          gmsh_yystringsymbols[argv[i]] = argv[i + 1];
+          Msg::GetCommandLineStrings()[argv[i]] = argv[i + 1];
+          i += 2;
+	}
+        else
+          Msg::Fatal("Missing name and/or value for string definition");
+      }
+      else if (!strcmp(argv[i]+1, "setnumber")) {
+        i++;
+	if (i + 1 < argc && argv[i][0] != '-') {
+          gmsh_yysymbols[argv[i]].value = std::vector<double>(1, atof(argv[i + 1]));
+          Msg::GetCommandLineNumbers()[argv[i]] = atof(argv[i + 1]);
+          i += 2;
+	}
+        else
+          Msg::Error("Missing name and/or value for number definition");
+      }
+#endif
       else if(!strcmp(argv[i] + 1, "option")) {
         i++;
         if(argv[i])
@@ -679,16 +703,6 @@ void GetOptions(int argc, char *argv[])
         else
           Msg::Fatal("Missing number");
       }
-      else if(!strcmp(argv[i] + 1, "mpass")) {
-        i++;
-        if(argv[i]) {
-          CTX::instance()->mesh.multiplePasses = atoi(argv[i++]);
-          if(CTX::instance()->mesh.multiplePasses <= 0)
-            Msg::Fatal("Number of Mesh Passes must be > 0");
-        }
-        else
-          Msg::Fatal("Missing number");
-      }
       else if(!strcmp(argv[i] + 1, "ignorePartBound")) {
         i++;
         opt_mesh_ignore_part_bound(0, GMSH_SET, 1);
@@ -792,8 +806,6 @@ void GetOptions(int argc, char *argv[])
             CTX::instance()->mesh.algo2d = ALGO_2D_FRONTAL_QUAD;
           else if(!strncmp(argv[i], "pack", 4))
             CTX::instance()->mesh.algo2d = ALGO_2D_PACK_PRLGRMS;
-          else if(!strncmp(argv[i], "ruppert", 7))
-            CTX::instance()->mesh.algo2d = ALGO_2D_RUPPERT;
           else if(!strncmp(argv[i], "front2d", 7) || !strncmp(argv[i], "frontal", 7))
             CTX::instance()->mesh.algo2d = ALGO_2D_FRONTAL;
           else if(!strncmp(argv[i], "bamg",4))
@@ -819,13 +831,11 @@ void GetOptions(int argc, char *argv[])
       }
       else if(!strcmp(argv[i] + 1, "rec")) {
         i++;
-        if(argv[i]) {
-          CTX::instance()->mesh.doRecombinationTest = 1;
-          CTX::instance()->mesh.recTestName = argv[i];
-          i++;
-        }
-        else
-          Msg::Fatal("Missing file name for recomb");
+        if (argc - i < 3) Msg::Fatal("pas assez argument");
+        CTX::instance()->mesh.doRecombinationTest = 1;
+        CTX::instance()->mesh.recTestName = argv[i]; i++;
+        CTX::instance()->mesh.nProc = atoi(argv[i]); i++;
+        CTX::instance()->mesh.nbProc = atoi(argv[i]); i++;
       }
       else if(!strcmp(argv[i] + 1, "beg")) {
         i++;
@@ -874,8 +884,38 @@ void GetOptions(int argc, char *argv[])
           Msg::Fatal("Missing format");
       }
       else if(!strcmp(argv[i] + 1, "listen")) {
-        CTX::instance()->solver.listen = 1;
         i++;
+#if !defined(HAVE_ONELAB2)
+        CTX::instance()->solver.listen = 1;
+#else
+        CTX::instance()->batch = 1;
+        if(argv[i]) {
+          CTX::instance()->onelab.listen_port = atoi(argv[i++]);
+        }
+        else
+          CTX::instance()->onelab.listen_port = 1148;
+#endif
+      }
+      else if(!strcmp(argv[i] + 1, "minterpreter")) {
+        i++;
+        if(argv[i])
+          CTX::instance()->solver.octaveInterpreter = argv[i++];
+        else
+          Msg::Fatal("Missing interpreter name");
+      }
+      else if(!strcmp(argv[i] + 1, "pyinterpreter")) {
+        i++;
+        if(argv[i])
+          CTX::instance()->solver.pythonInterpreter = argv[i++];
+        else
+          Msg::Fatal("Missing interpreter name");
+      }
+      else if(!strcmp(argv[i] + 1, "bg")){
+        i++;
+        if(argv[i])
+          CTX::instance()->bgImageFileName = argv[i++];
+        else
+          Msg::Fatal("Missing filename");
       }
       else if(!strcmp(argv[i] + 1, "version") || !strcmp(argv[i] + 1, "-version")) {
         fprintf(stderr, "%s\n", GMSH_VERSION);
@@ -883,32 +923,41 @@ void GetOptions(int argc, char *argv[])
       }
       else if(!strcmp(argv[i] + 1, "info") || !strcmp(argv[i] + 1, "-info")) {
         fprintf(stderr, "Version          : %s\n", GMSH_VERSION);
-#if defined(HAVE_FLTK)
-        fprintf(stderr, "GUI toolkit      : FLTK %d.%d.%d\n", FL_MAJOR_VERSION,
-                FL_MINOR_VERSION, FL_PATCH_VERSION);
-#else
-        fprintf(stderr, "GUI toolkit      : none\n");
-#endif
-#if defined(HAVE_PETSC)
-#if defined(PETSC_USE_COMPLEX)
-        fprintf(stderr, "PETSc arithmetic : Complex\n");
-#else
-        fprintf(stderr, "PETSc arithmetic : Real\n");
-#endif
-#endif
         fprintf(stderr, "License          : %s\n", GMSH_SHORT_LICENSE);
         fprintf(stderr, "Build OS         : %s\n", GMSH_OS);
-        fprintf(stderr, "Build options    :%s\n", GMSH_CONFIG_OPTIONS);
         fprintf(stderr, "Build date       : %s\n", GMSH_DATE);
         fprintf(stderr, "Build host       : %s\n", GMSH_HOST);
-        fprintf(stderr, "Packager         : %s\n", GMSH_PACKAGER);
+        fprintf(stderr, "Build options    :%s\n", GMSH_CONFIG_OPTIONS);
+#if defined(HAVE_FLTK)
+        fprintf(stderr, "FLTK version     : %d.%d.%d\n", FL_MAJOR_VERSION,
+                FL_MINOR_VERSION, FL_PATCH_VERSION);
+#endif
+#if defined(HAVE_PETSC)
+        fprintf(stderr, "PETSc version    : %d.%d.%d (%s arithmtic)\n", PETSC_VERSION_MAJOR,
+                PETSC_VERSION_MINOR, PETSC_VERSION_SUBMINOR,
+#if defined(PETSC_USE_COMPLEX)
+                "complex"
+#else
+                "real"
+#endif
+                );
+#endif
+#if defined(HAVE_OCC)
+        fprintf(stderr, "OCC version      : %d.%d.%d\n", OCC_VERSION_MAJOR,
+                OCC_VERSION_MINOR, OCC_VERSION_MAINTENANCE);
+#endif
+#if defined(HAVE_MED)
+        fprintf(stderr, "MED version      : %d.%d.%d\n", MED_NUM_MAJEUR,
+                MED_NUM_MINEUR, MED_NUM_RELEASE);
+#endif
+        fprintf(stderr, "Packaged by      : %s\n", GMSH_PACKAGER);
         fprintf(stderr, "Web site         : http://www.geuz.org/gmsh/\n");
         fprintf(stderr, "Mailing list     : gmsh@geuz.org\n");
         Msg::Exit(0);
       }
       else if(!strcmp(argv[i] + 1, "help") || !strcmp(argv[i] + 1, "-help")) {
         fprintf(stderr, "Gmsh, a 3D mesh generator with pre- and post-processing facilities\n");
-        fprintf(stderr, "Copyright (C) 1997-2013 Christophe Geuzaine and Jean-Francois Remacle\n");
+        fprintf(stderr, "Copyright (C) 1997-2015 Christophe Geuzaine and Jean-Francois Remacle\n");
         PrintUsage(argv[0]);
         Msg::Exit(0);
       }
