@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -53,10 +53,12 @@ StringXString GeneralOptions_String[] = {
     ".gmsh-errors" ,
 #endif
     "File into which the log is saved if a fatal error occurs" },
+  { F|S, "ExecutableFileName", opt_general_executable_filename , "" ,
+    "File name of the Gmsh executable (read-only)"},
 
   { F,   "FileName" , opt_general_filename , "" ,
     "Current project file name (read-only)" },
-  { F|O, "FltkTheme" , opt_general_gui_theme , "" ,
+  { F|S, "FltkTheme" , opt_general_gui_theme , "" ,
     "FLTK user interface theme (try e.g. plastic or gtk+)" },
 
   { F|O, "GraphicsFont" , opt_general_graphics_font , "Helvetica" ,
@@ -84,6 +86,16 @@ StringXString GeneralOptions_String[] = {
     "4th most recent opened file"},
   { F|S, "RecentFile4", opt_general_recent_file4 , "untitled.geo" ,
     "5th most recent opened file"},
+  { F|S, "RecentFile5", opt_general_recent_file5 , "untitled.geo" ,
+    "6th most recent opened file"},
+  { F|S, "RecentFile6", opt_general_recent_file6 , "untitled.geo" ,
+    "7th most recent opened file"},
+  { F|S, "RecentFile7", opt_general_recent_file7 , "untitled.geo" ,
+    "8th most recent opened file"},
+  { F|S, "RecentFile8", opt_general_recent_file8 , "untitled.geo" ,
+    "9th most recent opened file"},
+  { F|S, "RecentFile9", opt_general_recent_file9 , "untitled.geo" ,
+    "10th most recent opened file"},
 
   { 0,   "SessionFileName" , opt_general_session_filename ,
 #if defined(WIN32)
@@ -168,6 +180,11 @@ StringXString SolverOptions_String[] = {
   { F|S, "Name9" , opt_solver_name9 , "" ,
     "Name of solver 9" },
 
+  { F|S, "OctaveInterpreter" , opt_solver_octave_interpreter , "octave" ,
+    "Name of the Octave interpreter (used to run .m files)" },
+  { F|S, "PythonInterpreter" , opt_solver_python_interpreter , "python" ,
+    "Name of the Python interpreter (used to run .py files if they are not executable)" },
+
   { F|S, "RemoteLogin0" , opt_solver_remote_login0 , "",
     "Command to login to a remote host to launch solver 0" },
   { F|S, "RemoteLogin1" , opt_solver_remote_login1 , "" ,
@@ -204,6 +221,10 @@ StringXString SolverOptions_String[] = {
 } ;
 
 StringXString PostProcessingOptions_String[] = {
+  { F|O, "GraphPointCommand" , opt_post_graph_point_command, "" ,
+    "Command parsed when double-clicking on a graph data point "
+    "(e.g. Merge Sprintf('file_%g.pos', PostProcessing.GraphPointX);)" },
+
   { 0, 0 , 0 , "" , 0 }
 } ;
 
@@ -421,8 +442,8 @@ StringXNumber GeneralOptions_Number[] = {
     "planes window" },
   { F|O, "ClipWholeElements" , opt_general_clip_whole_elements , 0. ,
     "Clip whole elements" },
-  { F|O, "ColorScheme", opt_general_color_scheme , 1. ,
-    "Default color scheme (0=dark, 1=light or 2=grayscale)" },
+  { F|S, "ColorScheme", opt_general_color_scheme , 1. ,
+    "Default color scheme for graphics (0=dark, 1=light, 2=grayscale, 3=reverse)" },
   { F|O, "ConfirmOverwrite" , opt_general_confirm_overwrite, 1. ,
     "Ask confirmation before overwriting files?" },
   { F|S, "ContextPositionX" , opt_general_context_position0 , 650. ,
@@ -470,6 +491,8 @@ StringXNumber GeneralOptions_Number[] = {
   { F|S, "FileChooserPositionY" , opt_general_file_chooser_position1 , 200. ,
     "Vertical position (in pixels) of the upper left corner of the file chooser "
     "windows" },
+  { F|S, "FltkColorScheme" , opt_general_gui_color_scheme , 0. ,
+    "FLTK user interface color theme (0: standard, 1:dark)" },
   { F|O, "FontSize" , opt_general_fontsize , -1. ,
     "Size of the font in the user interface (-1=automatic)" },
 
@@ -494,6 +517,10 @@ StringXNumber GeneralOptions_Number[] = {
   { F|S, "HighOrderToolsPositionY" , opt_general_hot_position1 , 150. ,
     "Vertical position (in pixels) of the upper left corner of the high order "
     "tools window" },
+  { F|O, "HighResolutionGraphics" , opt_general_high_resolution_graphics , 1. ,
+    "Use high-resolution OpenGL graphics (e.g. for Macs with retina displays)" },
+  { F|O, "HighResolutionPointSizeFactor" , opt_general_high_resolution_point_size_factor , 2. ,
+    "Point size factor when using high-resolution OpenGL graphics" },
 
   { F|O, "InitialModule", opt_general_initial_context, 0. ,
     "Module launched on startup (0=automatic, 1=geometry, 2=mesh, 3=solver, "
@@ -588,6 +615,8 @@ StringXNumber GeneralOptions_Number[] = {
     "Horizontal position (in pixels) of the (detached) menu tree" },
   { F|S, "MenuPositionY" , opt_general_menu_position1 , 400. ,
     "Vertical position (in pixels) of the (detached) menu tree" },
+  { F|O, "MessageFontSize" , opt_general_message_fontsize , -1. ,
+    "Size of the font in the message window (-1=automatic)" },
   { F|S, "MessageHeight" , opt_general_message_size , 300. ,
     "Height (in pixels) of the message console when it is visible (should be > 0)" },
   { F,   "MinX" , opt_general_xmin , 0. ,
@@ -796,7 +825,9 @@ StringXNumber GeometryOptions_Number[] = {
   { F|O, "OCCSewFaces" , opt_geometry_occ_sew_faces , 0. ,
     "Sew faces in STEP, IGES and BRep models" },
   { F|O, "OCCConnectFaces" , opt_geometry_occ_connect_faces , 0. ,
-    "Cut and connect faces in STEP, IGES and BRep models" },
+    "Cut and connect faces in STEP, IGES and BRep models (0=no, 1=internal, 2=Salome)" },
+  { F|O, "OCCScaling" , opt_geometry_occ_scaling , 1. ,
+    "Scale STEP, IGES and BRep model by given factor" },
   { F,   "OffsetX" , opt_geometry_offset0 , 0. ,
     "Model display offset along X-axis (in model coordinates)" },
   { F,   "OffsetY" , opt_geometry_offset1 , 0. ,
@@ -1185,6 +1216,8 @@ StringXNumber MeshOptions_Number[] = {
   { F|O, "ToleranceEdgeLength" , opt_mesh_tolerance_edge_length, 0.0,
     "Skip a model edge in mesh generation if its length is less than user's "
     "defined tolerance" },
+  { F|O, "ToleranceInitialDelaunay" , opt_mesh_tolerance_initial_delaunay, 1.e-8,
+    "Tolerance for initial 3D Delaunay mesher" },
   { F|O, "Triangles" , opt_mesh_triangles , 1. ,
     "Display mesh triangles?" },
 
@@ -1247,12 +1280,16 @@ StringXNumber PostProcessingOptions_Number[] = {
 
   { F|O, "ForceNodeData" , opt_post_force_node_data , 0. ,
     "Try to force saving datasets as NodeData" },
-
   { F|O, "Format" , opt_post_file_format , 10. ,
     "Default file format for post-processing views (0=ASCII view, 1=binary "
     "view, 2=parsed view, 3=STL triangulation, 4=raw text, 5=Gmsh mesh, 6=MED file, "
     "10=automatic)" },
 
+  { F|O, "GraphPointX" , opt_post_graph_point_x , 0. ,
+    "Abscissa of last selected graph point" },
+  { F|O, "GraphPointY" , opt_post_graph_point_y , 0. ,
+    "Ordinate of last selected graph point" },
+
   { F|O, "HorizontalScales" , opt_post_horizontal_scales , 1. ,
     "Display value scales horizontally" },
 
@@ -1286,7 +1323,7 @@ StringXNumber ViewOptions_Number[] = {
   { F|O, "AutoPosition" , opt_view_auto_position , 1. ,
     "Position the scale or 2D plot automatically (0: manual, 1: automatic, 2: top left, "
     "3: top right, 4: bottom left, 5: bottom right, 6: top, 7: bottom, 8: left, 9: right, "
-    "10: full, 11: top third)" },
+    "10: full, 11: top third, 12: in model coordinates)" },
   { F|O, "Axes" , opt_view_axes , 0 ,
     "Axes (0=none, 1=simple axes, 2=box, 3=full grid, 4=open grid, 5=ruler)" },
   { F|O, "AxesMikado" , opt_view_axes_mikado , 0. ,
@@ -1316,7 +1353,7 @@ StringXNumber ViewOptions_Number[] = {
     "Draw the `N minus b'-dimensional boundary of the element (N=element "
     "dimension, b=option value)" },
 
-  { F|O, "CenterGlyphs" , opt_view_center_glyphs , 1,
+  { F|O, "CenterGlyphs" , opt_view_center_glyphs , 0,
     "Center glyphs (arrows, numbers, etc.)? (0=left, 1=centered, 2=right)" },
   { F,   "Clip" , opt_view_clip , 0.,
     "Enable clipping planes? (Plane[i]=2^i, i=0,...,5)" },
@@ -1432,6 +1469,8 @@ StringXNumber ViewOptions_Number[] = {
     "Maximum recursion level for adaptive views" },
   { F,   "Max" , opt_view_max , 0. ,
     "Maximum value in the view (read-only)" },
+  { F,   "MaxVisible" , opt_view_max_visible , 0. ,
+    "Maximum value in the visible parts of the view (read-only)" },
   { F,   "MaxX" , opt_view_xmax , 0. ,
     "Maximum view coordinate along the X-axis (read-only)" },
   { F,   "MaxY" , opt_view_ymax , 0. ,
@@ -1440,6 +1479,8 @@ StringXNumber ViewOptions_Number[] = {
     "Maximum view coordinate along the Z-axis (read-only)" },
   { F,   "Min" , opt_view_min , 0. ,
     "Minimum value in the view (read-only)" },
+  { F,   "MinVisible" , opt_view_min_visible , 0. ,
+    "Minimum value in the visible parts of the view (read-only)" },
   { F,   "MinX" , opt_view_xmin , 0. ,
     "Minimum view coordinate along the X-axis (read-only)" },
   { F,   "MinY" , opt_view_ymin , 0. ,
@@ -1510,6 +1551,9 @@ StringXNumber ViewOptions_Number[] = {
     "Tensor Visualization Type" },
   { F,   "TimeStep" , opt_view_timestep , 0. ,
     "Current time step displayed" },
+  { F,   "Time" , opt_view_time , -1. ,
+    "Current time displayed (if positive, sets the time step corresponding "
+    "the given time value)" },
   { F,   "TransformXX" , opt_view_transform00 , 1. ,
     "Element (1,1) of the 3x3 coordinate transformation matrix" },
   { F,   "TransformXY" , opt_view_transform01 , 0. ,
@@ -1529,7 +1573,7 @@ StringXNumber ViewOptions_Number[] = {
   { F,   "TransformZZ" , opt_view_transform22 , 1. ,
     "Element (3,3) of the 3x3 coordinate transformation matrix" },
   { F,   "Type" , opt_view_type , 1 ,
-    "Type of plot (1=3D, 2=2D space, 3=2D time)" },
+    "Type of plot (1=3D, 2=2D space, 3=2D time, 4=2D)" },
 
   { F|O, "UseGeneralizedRaise" , opt_view_use_gen_raise , 0 ,
     "Use generalized raise?" },
@@ -1652,70 +1696,70 @@ StringXNumber PrintOptions_Number[] = {
 
 StringXColor GeneralOptions_Color[] = {
   { F|O, "Background" , opt_general_color_background ,
-    {0, 0, 0, 255}, {255, 255, 255, 255}, {255, 255, 255, 255},
+    {245, 245, 245, 255}, {255, 255, 255, 255}, {245, 245, 245, 255}, {50, 50, 50, 255},
     "Background color" },
   { F|O, "BackgroundGradient" , opt_general_color_background_gradient ,
-    {0, 0, 128, 255}, {208, 215, 255, 255}, {125, 125, 125, 255},
+    {165, 165, 165, 255}, {208, 215, 255, 255}, {165, 165, 165, 255}, {50, 50, 50, 255},
     "Background gradient color" },
   { F|O, "Foreground" , opt_general_color_foreground ,
-    {170, 170, 170, 255}, {85, 85, 85, 255}, {128, 128, 128, 255},
+    {85, 85, 85, 255}, {85, 85, 85, 255}, {85, 85, 85, 255}, {170, 170, 170, 255},
     "Foreground color" },
   { F|O, "Text" , opt_general_color_text ,
-    {255, 255, 255, 255}, {0, 0, 0, 255}, {0, 0, 0, 255},
+    {0, 0, 0, 255}, {0, 0, 0, 255}, {0, 0, 0, 255}, {245, 245, 245, 255},
     "Text color" },
   { F|O, "Axes" , opt_general_color_axes ,
-    {255, 255, 255, 255}, {0, 0, 0, 255}, {0, 0, 0, 255},
+    {0, 0, 0, 255}, {0, 0, 0, 255}, {0, 0, 0, 255}, {245, 245, 245, 255},
     "Axes color" },
   { F|O, "SmallAxes" , opt_general_color_small_axes ,
-    {255, 255, 255, 255}, {0, 0, 0, 255}, {0, 0, 0, 255},
+    {0, 0, 0, 255}, {0, 0, 0, 255}, {0, 0, 0, 255}, {245, 245, 245, 255},
     "Small axes color" },
   { F|O, "AmbientLight" , opt_general_color_ambient_light,
-    {25, 25, 25, 255}, {25, 25, 25, 255}, {25, 25, 25, 255},
+    {25, 25, 25, 255}, {25, 25, 25, 255}, {25, 25, 25, 255}, {25, 25, 25, 255},
     "Ambient light color" },
   { F|O, "DiffuseLight" , opt_general_color_diffuse_light,
-    {255, 255, 255, 255}, {255, 255, 255, 255}, {255, 255, 255, 255},
+    {255, 255, 255, 255}, {255, 255, 255, 255}, {255, 255, 255, 255}, {255, 255, 255, 255},
     "Diffuse light color" },
   { F|O, "SpecularLight" , opt_general_color_specular_light,
-    {255, 255, 255, 255}, {255, 255, 255, 255}, {255, 255, 255, 255},
+    {255, 255, 255, 255}, {255, 255, 255, 255}, {255, 255, 255, 255}, {255, 255, 255, 255},
     "Specular light color" },
-  { 0, 0 , 0 ,  {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
+  { 0, 0 , 0 ,  {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , {0, 0, 0, 0} , 0 }
 } ;
 
 StringXColor GeometryOptions_Color[] = {
   { F|O, "Points" , opt_geometry_color_points ,
-    {178, 182, 129, 255}, {90, 90, 90, 255}, {0, 0, 0, 255},
+    {90, 90, 90, 255}, {90, 90, 90, 255}, {0, 0, 0, 255}, {178, 178, 178, 255},
     "Normal geometry point color" },
   { F|O, "Lines" , opt_geometry_color_lines ,
-    {0, 0, 255, 255}, {0, 0, 255, 255}, {0, 0, 0, 255},
+    {0, 0, 255, 255}, {0, 0, 255, 255}, {0, 0, 0, 255}, {0, 0, 255, 255},
     "Normal geometry curve color" },
   { F|O, "Surfaces" , opt_geometry_color_surfaces ,
-    {128, 128, 128, 255}, {128, 128, 128, 255}, {0, 0, 0, 255},
+    {128, 128, 128, 255}, {128, 128, 128, 255}, {0, 0, 0, 255}, {128, 128, 128, 255},
     "Normal geometry surface color" },
   { F|O, "Volumes" , opt_geometry_color_volumes ,
-    {255, 255, 0, 255}, {255, 255, 0, 255}, {0, 0, 0, 255},
+    {255, 255, 0, 255}, {255, 255, 0, 255}, {0, 0, 0, 255}, {255, 255, 0, 255},
     "Normal geometry volume color" },
   { F|O, "Selection" , opt_geometry_color_selection ,
-    {255, 0, 0, 255}, {255, 0, 0, 255}, {255, 0, 0, 255},
+    {255, 0, 0, 255}, {255, 0, 0, 255}, {255, 0, 0, 255}, {255, 0, 0, 255},
     "Selected geometry color" },
   { F|O, "HighlightZero" , opt_geometry_color_highlight0 ,
-    {255, 0, 0, 255}, {255, 0, 0, 255}, {255, 0, 0, 255},
+    {255, 0, 0, 255}, {255, 0, 0, 255}, {255, 0, 0, 255}, {255, 0, 0, 255},
     "Highlight 0 color" },
   { F|O, "HighlightOne" , opt_geometry_color_highlight1 ,
-    {255, 150, 0, 255}, {255, 150, 0, 255}, {255, 150, 0, 255},
+    {255, 150, 0, 255}, {255, 150, 0, 255}, {255, 150, 0, 255}, {255, 150, 0, 255},
     "Highlight 1 color" },
   { F|O, "HighlightTwo" , opt_geometry_color_highlight2 ,
-    {255, 255, 0, 255}, {255, 255, 0, 255}, {255, 255, 0, 255},
+    {255, 255, 0, 255}, {255, 255, 0, 255}, {255, 255, 0, 255}, {255, 255, 0, 255},
     "Highlight 2 color" },
   { F|O, "Tangents" , opt_geometry_color_tangents ,
-    {255, 255, 0, 255}, {255, 255, 0, 255}, {0, 0, 0, 255},
+    {255, 255, 0, 255}, {255, 255, 0, 255}, {0, 0, 0, 255}, {255, 255, 0, 255},
     "Tangent geometry vectors color" },
   { F|O, "Normals" , opt_geometry_color_normals ,
-    {255, 0, 0, 255}, {255, 0, 0, 255}, {0, 0, 0, 255},
+    {255, 0, 0, 255}, {255, 0, 0, 255}, {0, 0, 0, 255}, {255, 0, 0, 255},
     "Normal geometry vectors color" },
   { F|O, "Projection" , opt_geometry_color_projection ,
-    {0, 255, 0, 255}, {0, 255, 0, 255}, {0, 0, 0, 255},
+    {0, 255, 0, 255}, {0, 255, 0, 255}, {0, 0, 0, 255}, {0, 255, 0, 255},
     "Projection surface color" },
-  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
+  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
 } ;
 
 #define COLW  {210, 210, 210, 255}
@@ -1747,90 +1791,90 @@ StringXColor GeometryOptions_Color[] = {
 
 StringXColor MeshOptions_Color[] = {
   { F|O, "Points" , opt_mesh_color_points ,
-    {0, 0, 255, 255}, {0, 0, 255, 255}, {0, 0, 0, 255},
+    {0, 0, 255, 255}, {0, 0, 255, 255}, {0, 0, 0, 255}, {0, 0, 255, 255},
     "Mesh node color" },
   { F|O, "PointsSup" , opt_mesh_color_points_sup ,
-    {255, 0, 255, 255}, {255, 0, 255, 255}, {0, 0, 0, 255},
+    {255, 0, 255, 255}, {255, 0, 255, 255}, {0, 0, 0, 255}, {255, 0, 255, 255},
     "Second order mesh node color" },
   { F|O, "Lines" , opt_mesh_color_lines ,
-    {0, 0, 0, 255}, {0, 0, 0, 255}, {0, 0, 0, 255},
+    {0, 0, 0, 255}, {0, 0, 0, 255}, {0, 0, 0, 255}, {0, 0, 0, 255},
     "Mesh line color" },
   { F|O, "Triangles" , opt_mesh_color_triangles ,
-    COLT, COLT, COLW,
+    COLT, COLT, COLW, COLT,
     "Mesh triangle color (if Mesh.ColorCarousel=0)" },
   { F|O, "Quadrangles" , opt_mesh_color_quadrangles ,
-    COLQ, COLQ, COLW,
+    COLQ, COLQ, COLW, COLQ,
     "Mesh quadrangle color (if Mesh.ColorCarousel=0)" },
   { F|O, "Tetrahedra" , opt_mesh_color_tetrahedra ,
-    COLT, COLT, COLW,
+    COLT, COLT, COLW, COLT,
      "Mesh tetrahedron color (if Mesh.ColorCarousel=0)" },
   { F|O, "Hexahedra" , opt_mesh_color_hexahedra ,
-    COLQ, COLQ, COLW,
+    COLQ, COLQ, COLW, COLQ,
     "Mesh hexahedron color (if Mesh.ColorCarousel=0)" },
   { F|O, "Prisms" , opt_mesh_color_prisms ,
-    COLP, COLP, COLW,
+    COLP, COLP, COLW, COLP,
      "Mesh prism color (if Mesh.ColorCarousel=0)" },
   { F|O, "Pyramids" , opt_mesh_color_pyramid ,
-    COLY, COLY, COLW,
+    COLY, COLY, COLW, COLY,
      "Mesh pyramid color (if Mesh.ColorCarousel=0)" },
   { F|O, "Tangents" , opt_mesh_color_tangents ,
-    {255, 255, 0, 255}, {255, 255, 0, 255}, {0, 0, 0, 255},
+    {255, 255, 0, 255}, {255, 255, 0, 255}, {0, 0, 0, 255}, {255, 255, 0, 255},
     "Tangent mesh vector color" },
   { F|O, "Normals" , opt_mesh_color_normals ,
-    {255, 0, 0, 255}, {255, 0, 0, 255}, {0, 0, 0, 255},
+    {255, 0, 0, 255}, {255, 0, 0, 255}, {0, 0, 0, 255}, {255, 0, 0, 255},
     "Normal mesh vector color" },
-  { F|O, "Zero"     , opt_mesh_color_0 , COL0, COL0, COLW,
+  { F|O, "Zero"     , opt_mesh_color_0 , COL0, COL0, COLW, COL0,
     "Color 0 in color carousel" },
-  { F|O, "One"      , opt_mesh_color_1 , COL1, COL1, COLW,
+  { F|O, "One"      , opt_mesh_color_1 , COL1, COL1, COLW, COL1,
     "Color 1 in color carousel" },
-  { F|O, "Two"      , opt_mesh_color_2 , COL2, COL2, COLW,
+  { F|O, "Two"      , opt_mesh_color_2 , COL2, COL2, COLW, COL2,
     "Color 2 in color carousel" },
-  { F|O, "Three"    , opt_mesh_color_3 , COL3, COL3, COLW,
+  { F|O, "Three"    , opt_mesh_color_3 , COL3, COL3, COLW, COL3,
     "Color 3 in color carousel" },
-  { F|O, "Four"     , opt_mesh_color_4 , COL4, COL4, COLW,
+  { F|O, "Four"     , opt_mesh_color_4 , COL4, COL4, COLW, COL4,
     "Color 4 in color carousel" },
-  { F|O, "Five"     , opt_mesh_color_5 , COL5, COL5, COLW,
+  { F|O, "Five"     , opt_mesh_color_5 , COL5, COL5, COLW, COL5,
     "Color 5 in color carousel" },
-  { F|O, "Six"      , opt_mesh_color_6 , COL6, COL6, COLW,
+  { F|O, "Six"      , opt_mesh_color_6 , COL6, COL6, COLW, COL6,
     "Color 6 in color carousel" },
-  { F|O, "Seven"    , opt_mesh_color_7 , COL7, COL7, COLW,
+  { F|O, "Seven"    , opt_mesh_color_7 , COL7, COL7, COLW, COL7,
     "Color 7 in color carousel" },
-  { F|O, "Eight"    , opt_mesh_color_8 , COL8, COL8, COLW,
+  { F|O, "Eight"    , opt_mesh_color_8 , COL8, COL8, COLW, COL8,
     "Color 8 in color carousel" },
-  { F|O, "Nine"     , opt_mesh_color_9 , COL9, COL9, COLW,
+  { F|O, "Nine"     , opt_mesh_color_9 , COL9, COL9, COLW, COL9,
     "Color 9 in color carousel" },
-  { F|O, "Ten"      , opt_mesh_color_10, COL10, COL10, COLW,
+  { F|O, "Ten"      , opt_mesh_color_10, COL10, COL10, COLW, COL10,
     "Color 10 in color carousel" },
-  { F|O, "Eleven"   , opt_mesh_color_11, COL11, COL11, COLW,
+  { F|O, "Eleven"   , opt_mesh_color_11, COL11, COL11, COLW, COL11,
     "Color 11 in color carousel" },
-  { F|O, "Twelve"   , opt_mesh_color_12, COL12, COL12, COLW,
+  { F|O, "Twelve"   , opt_mesh_color_12, COL12, COL12, COLW, COL12,
     "Color 12 in color carousel" },
-  { F|O, "Thirteen" , opt_mesh_color_13, COL13, COL13, COLW,
+  { F|O, "Thirteen" , opt_mesh_color_13, COL13, COL13, COLW, COL13,
     "Color 13 in color carousel" },
-  { F|O, "Fourteen" , opt_mesh_color_14, COL14, COL14, COLW,
+  { F|O, "Fourteen" , opt_mesh_color_14, COL14, COL14, COLW, COL14,
     "Color 14 in color carousel" },
-  { F|O, "Fifteen"  , opt_mesh_color_15, COL15, COL15, COLW,
+  { F|O, "Fifteen"  , opt_mesh_color_15, COL15, COL15, COLW, COL15,
     "Color 15 in color carousel" },
-  { F|O, "Sixteen"  , opt_mesh_color_16, COL16, COL16, COLW,
+  { F|O, "Sixteen"  , opt_mesh_color_16, COL16, COL16, COLW, COL16,
     "Color 16 in color carousel" },
-  { F|O, "Seventeen", opt_mesh_color_17, COL17, COL17, COLW,
+  { F|O, "Seventeen", opt_mesh_color_17, COL17, COL17, COLW, COL17,
     "Color 17 in color carousel" },
-  { F|O, "Eighteen" , opt_mesh_color_18, COL18, COL18, COLW,
+  { F|O, "Eighteen" , opt_mesh_color_18, COL18, COL18, COLW, COL18,
     "Color 18 in color carousel" },
-  { F|O, "Nineteen" , opt_mesh_color_19, COL19, COL19, COLW,
+  { F|O, "Nineteen" , opt_mesh_color_19, COL19, COL19, COLW, COL19,
     "Color 19 in color carousel" },
-  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
+  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , {0, 0, 0, 0} , 0 }
 } ;
 
 StringXColor SolverOptions_Color[] = {
-  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
+  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
 } ;
 
 StringXColor PostProcessingOptions_Color[] = {
-  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
+  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
 } ;
 
-#define ELECOL  {255, 255, 255, 255}, {0, 0, 0, 255}, {0, 0, 0, 255}
+#define ELECOL  {0, 0, 0, 255}, {0, 0, 0, 255}, {0, 0, 0, 255}, {245, 245, 245, 255}
 
 StringXColor ViewOptions_Color[] = {
   { F|O, "Points" , opt_view_color_points , ELECOL, "Point color" },
@@ -1842,22 +1886,22 @@ StringXColor ViewOptions_Color[] = {
   { F|O, "Prisms" , opt_view_color_prisms , ELECOL, "Prism color" },
   { F|O, "Pyramids" , opt_view_color_pyramids , ELECOL, "Pyramid color" },
   { F|O, "Tangents" , opt_view_color_tangents ,
-    {255, 255, 0, 255}, {255, 255, 0, 255}, {0, 0, 0, 255},
+    {255, 255, 0, 255}, {255, 255, 0, 255}, {0, 0, 0, 255}, {255, 255, 0, 255},
     "Tangent vector color" },
   { F|O, "Normals" , opt_view_color_normals ,
-    {255, 0, 0, 255}, {255, 0, 0, 255}, {0, 0, 0, 255},
+    {255, 0, 0, 255}, {255, 0, 0, 255}, {0, 0, 0, 255}, {255, 0, 0, 255},
     "Normal vector color" },
   { F|O, "Text2D" , opt_view_color_text2d , ELECOL, "2D text color" },
   { F|O, "Text3D" , opt_view_color_text3d , ELECOL, "3D text color" },
   { F|O, "Axes" , opt_view_color_axes , ELECOL, "Axes color" },
   { F|O, "Background2D" , opt_view_color_background2d ,
-    {255, 255, 255, 200}, {255, 255, 255, 200}, {255, 255, 255, 200},
+    {255, 255, 255, 200}, {255, 255, 255, 200}, {255, 255, 255, 200}, {245, 245, 245, 200},
     "Bacground color for 2D plots" },
-  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
+  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
 } ;
 
 StringXColor PrintOptions_Color[] = {
-  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
+  { 0, 0 , 0 , {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0}, {0, 0, 0, 0} , 0 }
 } ;
 
 #undef S
