@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -25,10 +25,7 @@
 #include "PView.h"
 #endif
 
-//test new algo generation points
-#include "BasisFactory.h"
-
-#if defined(HAVE_ONELAB)
+#if defined(HAVE_ONELAB) && !defined(HAVE_ONELAB2)
 #include "gmshLocalNetworkClient.h"
 #endif
 
@@ -46,8 +43,16 @@
 #include "FlGui.h"
 #include "graphicWindow.h"
 #include "drawContext.h"
+#if defined(HAVE_ONELAB2)
+#include "onelab2Group.h"
+#else
 #include "onelabGroup.h"
 #endif
+#endif
+
+#if defined(HAVE_ONELAB2)
+#include "OnelabServer.h"
+#endif
 
 int GmshInitialize(int argc, char **argv)
 {
@@ -124,6 +129,24 @@ int GmshSetOption(const std::string &category, const std::string &name,
   return ColorOption(GMSH_SET|GMSH_GUI, category.c_str(), index, name.c_str(), value);
 }
 
+void GmshSetStringOption(const std::string &category, const std::string &name,
+                         std::string value, int index)
+{
+  StringOption(GMSH_SET|GMSH_GUI, category.c_str(), index, name.c_str(), value);
+}
+
+void GmshSetNumberOption(const std::string &category, const std::string &name,
+                         double value, int index)
+{
+  NumberOption(GMSH_SET|GMSH_GUI, category.c_str(), index, name.c_str(), value);
+}
+
+void GmshSetColorOption(const std::string &category, const std::string &name,
+                        unsigned int value, int index)
+{
+  ColorOption(GMSH_SET|GMSH_GUI, category.c_str(), index, name.c_str(), value);
+}
+
 int GmshGetOption(const std::string &category, const std::string &name,
                   std::string &value, int index)
 {
@@ -142,6 +165,30 @@ int GmshGetOption(const std::string &category, const std::string &name,
   return ColorOption(GMSH_GET, category.c_str(), index, name.c_str(), value);
 }
 
+std::string GmshGetStringOption(const std::string &category, const std::string &name,
+                                int index)
+{
+  std::string value;
+  StringOption(GMSH_GET, category.c_str(), index, name.c_str(), value);
+  return value;
+}
+
+double GmshGetNumberOption(const std::string &category, const std::string &name,
+                           int index)
+{
+  double value;
+  NumberOption(GMSH_GET, category.c_str(), index, name.c_str(), value);
+  return value;
+}
+
+int GmshGetColorOption(const std::string &category, const std::string &name,
+                       int index)
+{
+  unsigned int value;
+  ColorOption(GMSH_GET, category.c_str(), index, name.c_str(), value);
+  return value;
+}
+
 int GmshRestoreDefaultOptions()
 {
   ReInitOptions(0);
@@ -190,6 +237,13 @@ int GmshBatch()
             Msg::GetCommSize(), Msg::GetCommSize() > 1 ? "s" : "",
             Msg::GetMaxThreads(), Msg::GetMaxThreads() > 1 ? "s" : "");
   Msg::Info("Started on %s", Msg::GetLaunchDate().c_str());
+#if defined(HAVE_ONELAB2)
+  if(CTX::instance()->onelab.listen_port > 0) {
+    OnelabServer::instance(0, CTX::instance()->onelab.listen_port);
+    OnelabServer::instance()->Run();
+    Msg::Exit(0);
+  }
+#endif
 
   OpenProject(GModel::current()->getFileName());
   bool open = false;
@@ -207,6 +261,7 @@ int GmshBatch()
       MergeFile(CTX::instance()->files[i]);
   }
 
+
 #if defined(HAVE_POST) && defined(HAVE_MESH)
   if(!CTX::instance()->bgmFileName.empty()) {
     MergePostProcessingFile(CTX::instance()->bgmFileName);
@@ -218,7 +273,9 @@ int GmshBatch()
 #endif
 
   if(CTX::instance()->batch == -3){
+#if !defined(HAVE_ONELAB2)
     GmshRemote();
+#endif
   }
   else if(CTX::instance()->batch == -2){
     GModel::current()->checkMeshCoherence(CTX::instance()->geom.tolerance);
@@ -261,12 +318,11 @@ int GmshBatch()
     CreateOutputFile(name, CTX::instance()->mesh.fileFormat);
   }
 
-#if defined(HAVE_FLTK) // FIXME this actually does not require the GUI
   // launch solver (if requested)
-  solver_batch_cb(0, (void*)CTX::instance()->launchSolverAtStartup);
+#if defined(HAVE_ONELAB)
+  solver_batch_cb((void*)CTX::instance()->launchSolverAtStartup);
 #endif
 
-
   time_t now;
   time(&now);
   std::string currtime = ctime(&now);
@@ -333,12 +389,13 @@ int GmshFLTK(int argc, char **argv)
     else
       Msg::Error("Invalid background mesh (no view)");
   }
-
+#if !defined(HAVE_ONELAB2)
   // listen to external solvers
   if(CTX::instance()->solver.listen){
     gmshLocalNetworkClient *c = new gmshLocalNetworkClient("Listen", "");
     c->run();
   }
+#endif
 
   // launch solver (if requested) and fill onelab tree
   solver_cb(0, (void*)CTX::instance()->launchSolverAtStartup);
