@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -16,9 +16,11 @@
 #include <math.h>
 #include "GmshConfig.h"
 #include "StringUtils.h"
+#include "Context.h"
 
 #if defined(__APPLE__)
 #include <sys/sysctl.h>
+#include <mach-o/dyld.h>
 #endif
 
 #if defined(__linux__) && !defined(BUILD_ANDROID)
@@ -142,6 +144,68 @@ static unsigned utf8toUtf16(const char* src, unsigned srclen,
   return count;
 }
 
+static unsigned utf8FromUtf16(char* dst, unsigned dstlen,
+                              const wchar_t* src, unsigned srclen)
+{
+  unsigned i = 0;
+  unsigned count = 0;
+  if (dstlen) {
+    for (;;) {
+      unsigned ucs;
+      if (i >= srclen) {dst[count] = 0; return count;}
+      ucs = src[i++];
+      if (ucs < 0x80U) {
+        dst[count++] = ucs;
+        if (count >= dstlen) {dst[count-1] = 0; break;}
+      }
+      else if (ucs < 0x800U) { /* 2 bytes */
+        if (count+2 >= dstlen) {dst[count] = 0; count += 2; break;}
+        dst[count++] = 0xc0 | (ucs >> 6);
+        dst[count++] = 0x80 | (ucs & 0x3F);
+      }
+      else if (ucs >= 0xd800 && ucs <= 0xdbff && i < srclen &&
+	       src[i] >= 0xdc00 && src[i] <= 0xdfff) {
+        /* surrogate pair */
+        unsigned ucs2 = src[i++];
+        ucs = 0x10000U + ((ucs&0x3ff)<<10) + (ucs2&0x3ff);
+        /* all surrogate pairs turn into 4-byte utf8 */
+        if (count+4 >= dstlen) {dst[count] = 0; count += 4; break;}
+        dst[count++] = 0xf0 | (ucs >> 18);
+        dst[count++] = 0x80 | ((ucs >> 12) & 0x3F);
+        dst[count++] = 0x80 | ((ucs >> 6) & 0x3F);
+        dst[count++] = 0x80 | (ucs & 0x3F);
+      }
+      else {
+        /* all others are 3 bytes: */
+        if (count+3 >= dstlen) {dst[count] = 0; count += 3; break;}
+        dst[count++] = 0xe0 | (ucs >> 12);
+        dst[count++] = 0x80 | ((ucs >> 6) & 0x3F);
+        dst[count++] = 0x80 | (ucs & 0x3F);
+      }
+    }
+  }
+  /* we filled dst, measure the rest: */
+  while (i < srclen) {
+    unsigned ucs = src[i++];
+    if (ucs < 0x80U) {
+      count++;
+    }
+    else if (ucs < 0x800U) { /* 2 bytes */
+      count += 2;
+    }
+    else if (ucs >= 0xd800 && ucs <= 0xdbff && i < srclen-1 &&
+             src[i+1] >= 0xdc00 && src[i+1] <= 0xdfff) {
+      /* surrogate pair */
+      ++i;
+      count += 4;
+    }
+    else {
+      count += 3;
+    }
+  }
+  return count;
+}
+
 static wchar_t *wbuf[3] = {NULL, NULL, NULL};
 
 static void setwbuf(int i, const char *f)
@@ -186,7 +250,7 @@ const char *GetEnvironmentVar(const char *var)
 #endif
 }
 
-const void SetEnvironmentVar(const char *var, const char *val)
+void SetEnvironmentVar(const char *var, const char *val)
 {
 #if defined(WIN32) && !defined(__CYGWIN__)
   // should probably use Unicode version here
@@ -311,6 +375,35 @@ int GetProcessId()
 #endif
 }
 
+std::string GetExecutableFileName()
+{
+  std::string name = "";
+#if defined(WIN32) && !defined(__CYGWIN__)
+  WCHAR src[MAX_PATH];
+  DWORD size = GetModuleFileNameW(NULL, src, MAX_PATH);
+  if(size){
+    char dst[MAX_PATH];
+    utf8FromUtf16(dst, MAX_PATH, src, size);
+    name = std::string(dst);
+  }
+#elif defined(__APPLE__)
+  char path[PATH_MAX];
+  uint32_t size = sizeof(path);
+  if(_NSGetExecutablePath(path, &size) == 0){
+    char real[PATH_MAX];
+    if(realpath(path, real)){
+      name = std::string(real);
+    }
+  }
+#elif defined(__linux__)
+  char path[4096];
+  if(readlink("/proc/self/exe", path, 4096) > 0){
+    name = std::string(path);
+  }
+#endif
+  return name;
+}
+
 std::string GetHostName()
 {
   char host[256];
@@ -379,39 +472,36 @@ int KillProcess(int pid)
   return 1;
 }
 
-int SystemCall(const std::string &command, bool blocking)
+int SystemCallExe(const std::string &exe, const std::string &argsOrCommand,
+                  bool blocking)
 {
-  // separate (potential) executable from arguments
-  std::string exe, args;
-  std::string::size_type pos = command.find_first_of(" ");
-  if(pos != std::string::npos){
-    exe = command.substr(0, pos);
-    args = command.substr(pos, command.size() - pos);
-  }
-  else
-    exe = command;
-
-  // get executable extension
+  // do we try to run a .py script, .m script or an .exe?
   std::vector<std::string> split = SplitFileName(exe);
-
-  // do we try to run a .py script or a .exe?
   bool isPython = (split[2] == ".py" || split[2] == ".PY");
+  bool isOctave = (split[2] == ".m" || split[2] == ".M");
   bool isExe = (split[2] == ".exe" || split[2] == ".EXE");
 
-  if(isPython || isExe){
+  if(isPython || isOctave || isExe){
     if(StatFile(exe)){
       Msg::Error("Unable to open file '%s'", exe.c_str());
       return 1;
     }
   }
 
+  std::string command;
+  if(exe.size()){
+    command.append("\"" + exe + "\""); // allows exe with white space
+    if(argsOrCommand.size()) command.append(" ");
+  }
+  command.append(argsOrCommand);
+
 #if defined(WIN32) && !defined(__CYGWIN__)
-  if(isPython){
+  if(isPython || isOctave){
     Msg::Info("Shell opening '%s' with arguments '%s'", exe.c_str(),
-              args.c_str());
+              argsOrCommand.c_str());
     setwbuf(0, "open");
     setwbuf(1, exe.c_str());
-    setwbuf(2, args.c_str());
+    setwbuf(2, argsOrCommand.c_str());
     ShellExecuteW(NULL, wbuf[0], wbuf[1], wbuf[2], NULL, 0);
   }
   else{
@@ -441,12 +531,17 @@ int SystemCall(const std::string &command, bool blocking)
   }
 #else
   std::string cmd(command);
-  if(isPython || isExe){
+  if(isPython || isOctave || isExe){
     if(access(exe.c_str(), X_OK)){
       if(isPython){
-        Msg::Info("Script '%s' is not executable: running with python",
-		  exe.c_str());
-        cmd = "python " + cmd;
+        Msg::Info("Script '%s' is not executable: running with `%s'",
+		  exe.c_str(), CTX::instance()->solver.pythonInterpreter.c_str());
+        cmd = CTX::instance()->solver.pythonInterpreter + " " + cmd;
+      }
+      else if(isOctave){
+        Msg::Info("Script '%s' is not executable: running with `%s'",
+		  exe.c_str(), CTX::instance()->solver.octaveInterpreter.c_str());
+        cmd = CTX::instance()->solver.octaveInterpreter + " " + cmd;
       }
       else
         Msg::Warning("File '%s' is not executable", exe.c_str());
@@ -467,6 +562,11 @@ int SystemCall(const std::string &command, bool blocking)
   return 0;
 }
 
+int SystemCall(const std::string &command, bool blocking)
+{
+  return SystemCallExe("", command, blocking);
+}
+
 std::string GetCurrentWorkdir()
 {
   char path[1024];
