@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -22,10 +22,87 @@
 #include "PViewOptions.h"
 #endif
 
+#if defined(HAVE_ONELAB2)
+#include "OnelabDatabase.h"
+#include "OnelabServer.h"
+#endif
+
 namespace onelabUtils {
 
   // get command line arguments for the client if "UseCommandLine" is set for
   // this client
+#ifdef HAVE_ONELAB2
+  std::vector<std::string> getCommandLine(const std::string &client)
+  {
+    std::vector<std::string> args;
+    std::vector<onelab::number> n;
+    OnelabDatabase::instance()->get(n, client + "/UseCommandLine");
+    if(n.size() && n[0].getValue()){
+      std::vector<onelab::string> ps;
+      OnelabDatabase::instance()->get(ps, client + "/Action");
+      std::string action = (ps.empty() ? "" : ps[0].getValue());
+      OnelabDatabase::instance()->get(ps, client + "/1ModelName");
+      std::string modelName = (ps.empty() ? "" : ps[0].getValue());
+      OnelabDatabase::instance()->get(ps, client + "/9CheckCommand");
+      std::string checkCommand = (ps.empty() ? "" : ps[0].getValue());
+      OnelabDatabase::instance()->get(ps, client + "/9ComputeCommand");
+      std::string computeCommand = (ps.empty() ? "" : ps[0].getValue());
+      if(modelName.size()) args.push_back(" \"" + modelName + "\"");
+      if(action == "check")
+        args.push_back(" " + checkCommand) ;
+      else if(action == "compute")
+        args.push_back(" " + computeCommand);
+    }
+    return args;
+  }
+  std::string getMshFileName()
+  {
+    std::string name;
+    std::vector<onelab::string> ps;
+    OnelabDatabase::instance()->get(ps, "Gmsh/MshFileName");
+    if(ps.size()){
+      name = ps[0].getValue();
+    }
+    else{
+      name = CTX::instance()->outputFileName;
+      if(name.empty()){
+        if(CTX::instance()->mesh.fileFormat == FORMAT_AUTO)
+          name = GetDefaultFileName(FORMAT_MSH);
+        else
+          name = GetDefaultFileName(CTX::instance()->mesh.fileFormat);
+      }
+      onelab::string o("Gmsh/MshFileName", name, "Mesh name");
+      o.setKind("file");
+      o.setAttribute("Closed", "1");
+      OnelabDatabase::instance()->set(o, std::string("Gmsh"));
+    }
+    return name;
+  }
+
+  void guessModelName(const std::string &client)
+  {
+    std::vector<onelab::number> n;
+    OnelabDatabase::instance()->get(n, client + "/GuessModelName");
+    if(n.size() && n[0].getValue()){
+      std::vector<onelab::string> ps;
+      OnelabDatabase::instance()->get(ps, client + "/1ModelName");
+      if(ps.empty()){
+        OnelabDatabase::instance()->get(ps, "Gmsh/1ModelName");
+        std::vector<std::string> split;
+        if(!ps.empty()) split = SplitFileName(ps[0].getValue());
+        else split = SplitFileName(GModel::current()->getFileName());
+        std::string ext = "";
+        OnelabDatabase::instance()->get(ps, client + "/FileExtension");
+        if(ps.size()) ext = ps[0].getValue();
+        std::string name(split[0] + split[1] + ext);
+        onelab::string o(client + "/1ModelName", name, "Model name");
+        o.setKind("file");
+        OnelabDatabase::instance()->set(o, client);
+      }
+    }
+  }
+
+#else
   std::vector<std::string> getCommandLine(onelab::client *c)
   {
     std::vector<std::string> args;
@@ -47,12 +124,23 @@ namespace onelabUtils {
         args.push_back(" " + checkCommand) ;
       else if(action == "compute")
         args.push_back(" " + computeCommand);
+      // Propagate -setnumber/-setnumber gmsh option to the client. (Is this a
+      // good idea?)
+      std::ostringstream sstream;
+      sstream.precision(16);
+      std::map<std::string, double> cln(Msg::GetCommandLineNumbers());
+      for(std::map<std::string, double>::iterator it = cln.begin(); it != cln.end(); it++)
+        sstream << " -setnumber " << it->first << " " << it->second;
+      std::map<std::string, std::string> cls(Msg::GetCommandLineStrings());
+      for(std::map<std::string, std::string>::iterator it = cls.begin(); it != cls.end(); it++)
+        sstream << " -setstring " << it->first << " " << it->second;
+      args.push_back(sstream.str());
     }
     return args;
   }
-
   std::string getMshFileName(onelab::client *c)
   {
+
     std::string name;
     std::vector<onelab::string> ps;
     c->get(ps, "Gmsh/MshFileName");
@@ -104,6 +192,8 @@ namespace onelabUtils {
     }
   }
 
+#endif
+
   void initializeLoop(const std::string &level)
   {
     bool changed = false;
@@ -243,18 +333,22 @@ namespace onelabUtils {
     int num = atoi(graphNum.c_str());
     std::vector<double> x, y;
     std::string xName, yName;
+    int graphType = 3;
     std::vector<onelab::number> numbers;
     onelab::server::instance()->get(numbers);
     for(unsigned int i = 0; i < numbers.size(); i++){
       std::string v = numbers[i].getAttribute("Graph");
       v.resize(36, '0');
-      if(v[2 * num] == '1'){
+      if(v[2 * num] != '0'){
         x = getRange(numbers[i]);
         xName = numbers[i].getShortName();
       }
-      if(v[2 * num + 1] == '1'){
+      if(v[2 * num + 1] != '0'){
         y = getRange(numbers[i]);
         yName = numbers[i].getShortName();
+        char t = v[2 * num + 1];
+        graphType = (t == '1') ? 1 : (t == '2') ? 2 : (t == '3') ? 3 :
+          (t == '4') ? 4 : 3;
       }
     }
     if(x.empty()){
@@ -263,8 +357,8 @@ namespace onelabUtils {
     }
     if(x.size() && y.size()){
       if(x.size() != y.size())
-        Msg::Warning("X-Y data series have different length (%d != %d)",
-                     (int)x.size(), (int)y.size());
+        Msg::Info("X-Y data series have different length (%d != %d)",
+                  (int)x.size(), (int)y.size());
       if(view){
         view->getData()->setXY(x, y);
         view->getData()->setName(yName);
@@ -274,7 +368,7 @@ namespace onelabUtils {
       else{
         view = new PView(xName, yName, x, y);
         view->getData()->setFileName("ONELAB" + graphNum);
-        view->getOptions()->intervalsType = PViewOptions::Discrete;
+        view->getOptions()->intervalsType = graphType;
         view->getOptions()->autoPosition = num / 2 + 2;
       }
       changed = true;
@@ -290,7 +384,78 @@ namespace onelabUtils {
   static bool _firstComputation = true;
   void setFirstComputationFlag(bool val){ _firstComputation = val; }
   bool getFirstComputationFlag(){ return _firstComputation; }
+#ifdef HAVE_ONELAB2
+  bool runGmshClient(const std::string &action, int meshAuto)
+  {
+    bool redraw = false;
 
+    // do nothing in case of a python metamodel
+    std::vector<onelab::number> pn;
+    OnelabDatabase::instance()->get(pn, "IsPyMetamodel");
+    if(pn.size() && pn[0].getValue()) return redraw;
+
+    OnelabLocalNetworkClient *c = OnelabServer::instance()->getClient("Gmsh");
+    if(c != NULL) return redraw; // Gmsh is remote TODO
+
+    std::string mshFileName = onelabUtils::getMshFileName();
+
+    Msg::SetGmshOnelabAction(action);
+
+    static std::string modelName = GModel::current()->getName();
+
+    if(action == "initialize"){
+      // nothing to do
+    }
+    else if(action == "reset"){
+      setFirstComputationFlag(false);
+      // nothing more to do: "check" will be called right afterwards
+    }
+    else if(action == "check"){
+      if(OnelabServer::instance()->getChanged("Gmsh") ||
+         modelName != GModel::current()->getName()){
+        // reload geometry if Gmsh parameters have been modified or
+        // if the model name has changed
+        modelName = GModel::current()->getName();
+        redraw = true;
+        OpenProject(GModel::current()->getFileName(), false);
+        OnelabServer::instance()->setChanged(true, "Gmsh");
+      }
+    }
+    else if(action == "compute"){
+      if(OnelabServer::instance()->getChanged("Gmsh") ||
+         modelName != GModel::current()->getName()){
+        // reload the geometry, mesh it and save the mesh if Gmsh parameters
+        // have been modified or if the model name has changed
+        modelName = GModel::current()->getName();
+        redraw = true;
+        OpenProject(GModel::current()->getFileName(), false);
+        if(getFirstComputationFlag() && !StatFile(mshFileName) && meshAuto != 2){
+          Msg::Info("Skipping mesh generation: assuming '%s' is up-to-date "
+                    "(use Solver.AutoMesh=2 to force mesh generation)",
+                    mshFileName.c_str());
+        }
+        else if(!GModel::current()->empty() && meshAuto){
+          GModel::current()->mesh(3);
+          CreateOutputFile(mshFileName, CTX::instance()->mesh.fileFormat);
+        }
+      }
+      else if(StatFile(mshFileName)){
+        // mesh+save if the mesh file does not exist
+        if(meshAuto){
+          redraw = true;
+          GModel::current()->mesh(3);
+          CreateOutputFile(mshFileName, CTX::instance()->mesh.fileFormat);
+        }
+      }
+      setFirstComputationFlag(false);
+      OnelabServer::instance()->setChanged(false, "Gmsh");
+    }
+
+    Msg::SetGmshOnelabAction("");
+
+    return redraw;
+  }
+#else
   bool runGmshClient(const std::string &action, int meshAuto)
   {
     bool redraw = false;
@@ -303,7 +468,7 @@ namespace onelabUtils {
     onelab::server::citer it = onelab::server::instance()->findClient("Gmsh");
     if(it == onelab::server::instance()->lastClient()) return redraw;
 
-    onelab::client *c = it->second;
+    onelab::client *c = *it;
     std::string mshFileName = onelabUtils::getMshFileName(c);
 
     Msg::SetGmshOnelabAction(action);
@@ -361,7 +526,7 @@ namespace onelabUtils {
 
     return redraw;
   }
-
+#endif
   // update x using y, giving priority to any settings in x that can be set in
   // the GUI. The value of x is only changed if y is read-only.
   double updateNumber(onelab::number &x, onelab::number &y, const bool readOnlyRange)
