@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -8,6 +8,7 @@
 
 #include <string>
 #include <vector>
+#include "SPoint2.h"
 
 #define GMSH_WINDOW_BOX FL_FLAT_BOX
 #define GMSH_SIMPLE_RIGHT_BOX (Fl_Boxtype)(FL_FREE_BOXTYPE+1)
@@ -44,6 +45,7 @@ class FlGui{
  private:
   static FlGui *_instance;
   static std::string _openedThroughMacFinder;
+  int _in_main_thread;
   std::string _lastStatus;
  public:
   std::vector<GVertex*> selectedVertices;
@@ -51,6 +53,7 @@ class FlGui{
   std::vector<GFace*> selectedFaces;
   std::vector<GRegion*> selectedRegions;
   std::vector<MElement*> selectedElements;
+  std::vector<SPoint2> selectedPoints;
  public:
   std::vector<graphicWindow*> graph;
   optionWindow *options;
@@ -81,6 +84,9 @@ class FlGui{
   static void wait();
   // wait (at most time seconds) for any events, then process them
   static void wait(double time);
+  int lock();
+  void unlock();
+  bool in_main_thread() {return _in_main_thread==0;}
   // is a file opened through the Mac Finder?
   static void setOpenedThroughMacFinder(const std::string &name)
   {
@@ -117,8 +123,6 @@ class FlGui{
   void setLastStatus(int col=-1);
   // display status message and update progress bar
   void setProgress(const std::string &msg, double val, double min, double max);
-  // set color of progress message
-  void setProgressColor(int col);
   // create the window for physical context dependant definitions
   void callForSolverPlugin(int dim);
   // add line in message console
@@ -129,6 +133,8 @@ class FlGui{
   void rebuildTree(bool deleteWidgets);
   // open module in tree
   void openModule(const std::string &name);
+  // apply color scheme to widgets
+  void applyColorScheme();
 };
 
 void redraw_cb(Fl_Widget *w, void *data);
