@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -17,6 +17,7 @@
 #include "MElement.h"
 #include "Numeric.h"
 #include "FlGui.h"
+#include "OpenFile.h"
 #include "drawContext.h"
 #include "Context.h"
 #include "Trackball.h"
@@ -41,7 +42,6 @@ static void navigator_handler(void *data)
   }
 }
 
-
 static void lassoZoom(drawContext *ctx, mousePosition &click1, mousePosition &click2)
 {
   if(click1.win[0] == click2.win[0] || click1.win[1] == click2.win[1]) return;
@@ -65,7 +65,8 @@ openglWindow::openglWindow(int x, int y, int w, int h)
   :  Fl_Gl_Window(x, y, w, h, "gl"), _lock(false), _drawn(false),
      _selection(ENT_NONE), _trySelection(0), Nautilus(0)
 {
-  _ctx = new drawContext();
+  _ctx = new drawContext(this);
+
   for(int i = 0; i < 3; i++) _point[i] = 0.;
   for(int i = 0; i < 4; i++) _trySelectionXYWH[i] = 0;
   _lassoXY[0] = _lassoXY[1] = 0;
@@ -73,7 +74,7 @@ openglWindow::openglWindow(int x, int y, int w, int h)
   addPointMode = lassoMode = selectionMode = false;
   endSelection = undoSelection = invertSelection = quitSelection = 0;
 
-  if(CTX::instance()->gamepad)   Fl::add_timeout(.5, navigator_handler, (void*)this);
+  if(CTX::instance()->gamepad) Fl::add_timeout(.5, navigator_handler, (void*)this);
 }
 
 openglWindow::~openglWindow()
@@ -156,8 +157,7 @@ void openglWindow::draw()
   _ctx->viewport[1] = 0;
   _ctx->viewport[2] = w();
   _ctx->viewport[3] = h();
-  glViewport(_ctx->viewport[0], _ctx->viewport[1],
-             _ctx->viewport[2], _ctx->viewport[3]);
+  glViewport(0, 0, pixel_w(), pixel_h());
 
   if(lassoMode) {
     // draw the zoom or selection lasso on top of the current scene (without
@@ -215,7 +215,10 @@ void openglWindow::draw()
 
     _ctx->draw3d();
     glColor4ubv((GLubyte *) & CTX::instance()->color.fg);
-    glPointSize((float)CTX::instance()->geom.pointSize);
+    float ps = CTX::instance()->geom.pointSize;
+    if(_ctx->isHighResolution())
+      ps *= CTX::instance()->highResolutionPointSizeFactor;
+    glPointSize(ps);
     glBegin(GL_POINTS);
     glVertex3d(_point[0], _point[1], _point[2]);
     glEnd();
@@ -240,12 +243,12 @@ void openglWindow::draw()
     if(CTX::instance()->camera && !CTX::instance()->stereo){
       Camera *cam = &(_ctx->camera);
       if (!cam->on) cam->init();
-      cam->giveViewportDimension(_ctx->viewport[2],_ctx->viewport[3]);
+      cam->giveViewportDimension(_ctx->viewport[2], _ctx->viewport[3]);
       glMatrixMode(GL_PROJECTION);
       glLoadIdentity();
 
       glFrustum(cam->glFleft, cam->glFright, cam->glFbottom,
-                cam->glFtop, cam->glFnear , cam->glFfar*cam->Lc );
+                cam->glFtop, cam->glFnear, cam->glFfar*cam->Lc);
 
       glMatrixMode(GL_MODELVIEW);
       glLoadIdentity();
@@ -344,7 +347,23 @@ int openglWindow::handle(int event)
   case FL_PUSH:
     if(Fl::event_clicks() == 1 && !selectionMode){
       // double-click and not in selection mode
-      status_options_cb(0, (void*)"quick_access");
+      std::vector<GVertex*> vertices;
+      std::vector<GEdge*> edges;
+      std::vector<GFace*> faces;
+      std::vector<GRegion*> regions;
+      std::vector<MElement*> elements;
+      std::vector<SPoint2> points;
+      _select(ENT_ALL, false, false, Fl::event_x(), Fl::event_y(), 5, 5,
+              vertices, edges, faces, regions, elements, points);
+      if(points.size()){ // double-click on graph point
+        CTX::instance()->post.graphPointX = points[0].x();
+        CTX::instance()->post.graphPointY = points[0].y();
+        if(CTX::instance()->post.graphPointCommand.size())
+          ParseString(CTX::instance()->post.graphPointCommand, true);
+      }
+      else{ // popup quick access menu
+        status_options_cb(0, (void*)"quick_access");
+      }
       Fl::event_clicks(-1);
       return 1;
     }
@@ -609,9 +628,10 @@ int openglWindow::handle(int event)
         std::vector<GFace*> faces;
         std::vector<GRegion*> regions;
         std::vector<MElement*> elements;
+        std::vector<SPoint2> points;
         bool res = _select(_selection, false, CTX::instance()->mouseHoverMeshes,
                            (int)_curr.win[0], (int)_curr.win[1], 5, 5,
-                           vertices, edges, faces, regions, elements);
+                           vertices, edges, faces, regions, elements, points);
         if((_selection == ENT_ALL && res) ||
            (_selection == ENT_POINT && vertices.size()) ||
            (_selection == ENT_LINE && edges.size()) ||
@@ -629,6 +649,17 @@ int openglWindow::handle(int event)
         std::string text;
         if(ge) text += ge->getInfoString();
         if(me) text += me->getInfoString();
+        if(points.size()){
+          char tmp[256];
+          sprintf(tmp, "Point (%g, %g)", points[0].x(), points[0].y());
+          text += tmp;
+          if(CTX::instance()->post.graphPointCommand.size()){
+            text += std::string(" - Double-click to execute\n\n");
+            std::string cmd = CTX::instance()->post.graphPointCommand;
+            std::replace(cmd.begin(), cmd.end(), '\r', ' ');
+            text += cmd;
+          }
+        }
         if(CTX::instance()->tooltips)
           drawTooltip(text);
         else
@@ -643,13 +674,32 @@ int openglWindow::handle(int event)
   }
 }
 
+int openglWindow::pixel_w()
+{
+#if (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION == 3) && (FL_PATCH_VERSION >= 4)
+  return Fl_Gl_Window::pixel_w();
+#else
+  return w();
+#endif
+}
+
+int openglWindow::pixel_h()
+{
+#if (FL_MAJOR_VERSION == 1) && (FL_MINOR_VERSION == 3) && (FL_PATCH_VERSION >= 4)
+  return Fl_Gl_Window::pixel_h();
+#else
+  return h();
+#endif
+}
+
 bool openglWindow::_select(int type, bool multiple, bool mesh,
                            int x, int y, int w, int h,
                            std::vector<GVertex*> &vertices,
                            std::vector<GEdge*> &edges,
                            std::vector<GFace*> &faces,
                            std::vector<GRegion*> &regions,
-                           std::vector<MElement*> &elements)
+                           std::vector<MElement*> &elements,
+                           std::vector<SPoint2> &points)
 {
   // same lock as in draw() to prevent firing up a GL_SELECT rendering pass
   // while a GL_RENDER pass is happening (due to the asynchronus nature of
@@ -658,7 +708,8 @@ bool openglWindow::_select(int type, bool multiple, bool mesh,
   _lock = true;
   make_current();
   bool ret = _ctx->select(type, multiple, mesh, x, y, w, h,
-                          vertices, edges, faces, regions, elements);
+                          vertices, edges, faces, regions, elements,
+                          points);
   _lock = false;
   return ret;
 }
@@ -668,7 +719,8 @@ char openglWindow::selectEntity(int type,
                                 std::vector<GEdge*> &edges,
                                 std::vector<GFace*> &faces,
                                 std::vector<GRegion*> &regions,
-                                std::vector<MElement*> &elements)
+                                std::vector<MElement*> &elements,
+                                std::vector<SPoint2> &points)
 {
   // force keyboard focus in GL window
   take_focus();
@@ -719,7 +771,7 @@ char openglWindow::selectEntity(int type,
       else if(_select(_selection, multi, true, _trySelectionXYWH[0],
                       _trySelectionXYWH[1], _trySelectionXYWH[2],
                       _trySelectionXYWH[3], vertices, edges, faces,
-                      regions, elements)){
+                      regions, elements, points)){
         _selection = ENT_NONE;
         selectionMode = false;
         if(add)
