@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -176,6 +176,8 @@ void GEdge::setVisibility(char val, bool recursive)
 std::string GEdge::getAdditionalInfoString()
 {
   std::ostringstream sstream;
+  sstream.precision(12);
+
   if(v0 && v1) sstream << "{" << v0->tag() << " " << v1->tag() << "}";
 
   if(meshAttributes.method == MESH_TRANSFINITE){
@@ -445,12 +447,12 @@ void GEdge::replaceEndingPoints(GVertex *replOfv0, GVertex *replOfv1)
 {
   replaceEndingPointsInternals(replOfv0, replOfv1);
   if (replOfv0 != v0){
-    v0->delEdge(this);
+    if (v0) v0->delEdge(this);
     replOfv0->addEdge(this);
     v0 = replOfv0;
   }
   if (replOfv1 != v1){
-    v1->delEdge(this);
+    if (v1) v1->delEdge(this);
     replOfv1->addEdge(this);
     v1 = replOfv1;
   }
