@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -22,7 +22,7 @@
 #if defined(HAVE_MESH)
 #include "meshGFaceOptimize.h"
 #include "meshGFaceLloyd.h"
-#include "BackgroundMesh.h"
+#include "BackgroundMeshTools.h"
 #endif
 
 #if defined(HAVE_BFGS)
@@ -107,6 +107,23 @@ void GFace::delFreeEdge(GEdge *e)
   }
 }
 
+void GFace::replaceEdge(GEdge *e1, GEdge *e2)
+{
+  std::list<GEdge*>::iterator ite = l_edges.begin();
+  std::list<GEdge*> newlist;
+  newlist.clear();
+  while(ite != l_edges.end()){
+    if(e1 == *ite){
+      newlist.push_back(e2);
+    }
+    else{
+      newlist.push_back((*ite));
+    }
+    ite++;
+  }
+  l_edges = newlist;
+}
+
 void GFace::deleteMesh()
 {
   for(unsigned int i = 0; i < mesh_vertices.size(); i++) delete mesh_vertices[i];
@@ -259,6 +276,26 @@ surface_params GFace::getSurfaceParams() const
   return p;
 }
 
+std::vector<MVertex*> GFace::getEmbeddedMeshVertices() const
+{
+  std::set<MVertex*> tmp;
+  for(std::list<GEdge *>::const_iterator it = embedded_edges.begin();
+      it != embedded_edges.end(); it++){
+    tmp.insert((*it)->mesh_vertices.begin(), (*it)->mesh_vertices.end());
+    tmp.insert((*it)->getBeginVertex()->mesh_vertices.begin(),
+               (*it)->getBeginVertex()->mesh_vertices.end());
+    tmp.insert((*it)->getEndVertex()->mesh_vertices.begin(),
+               (*it)->getEndVertex()->mesh_vertices.end());
+  }
+  for(std::list<GVertex *>::const_iterator it = embedded_vertices.begin();
+      it != embedded_vertices.end(); it++){
+    tmp.insert((*it)->mesh_vertices.begin(), (*it)->mesh_vertices.end());
+  }
+  std::vector<MVertex*> res;
+  res.insert(res.end(), tmp.begin(), tmp.end());
+  return res;
+}
+
 std::list<GVertex*> GFace::vertices() const
 {
   std::set<GVertex*> v;
@@ -958,110 +995,61 @@ void bfgs_callback(const alglib::real_1d_array& x, double& func,
 GPoint GFace::closestPoint(const SPoint3 &queryPoint, const double initialGuess[2]) const
 {
 #if defined(HAVE_BFGS)
-  // Creating the optimisation problem
-  // printf("STARTING OPTIMIZATION\n");
+  // Test initial guess
+  double min_u = initialGuess[0];
+  double min_v = initialGuess[1];
+  GPoint pnt = point(min_u, min_v);
+  SPoint3 spnt(pnt.x(), pnt.y(), pnt.z());
+  double min_dist = queryPoint.distance(spnt);
 
-  alglib::ae_int_t dim = 2;
-  alglib::ae_int_t corr = 2; // Num of corrections in the scheme in [3,7]
-  alglib::minlbfgsstate state;
-  alglib::real_1d_array x; // = "[0.5,0.5]";
-  std::vector<double> initial_conditions(dim);
-
-  double min_dist = 1e18;
-  double min_u = 0.5;
-  double min_v = 0.5;
-
-  Range<double> uu = parBounds(0);
-  Range<double> vv = parBounds(1);
-  {
-    GPoint pnt = point(initialGuess[0], initialGuess[1]);
-    SPoint3 spnt(pnt.x(), pnt.y(), pnt.z());
-    //double dist = queryPoint.distance(spnt);
-  }
-
-  // FILE *F = Fopen ("hop.pos","w");
-  // fprintf(F,"View \" \" {\n");
-  // fprintf(F,"SP(%g,%g,%g) {%g};\n",queryPoint.x(),queryPoint.y(),queryPoint.z(),0.0);
-  double initial_guesses = 10.0;
-  for(double u = uu.low(); u <= uu.high() + 1.e-5;
-      u += (uu.high() - uu.low()) / initial_guesses) {
-    //    printf("%f\n", u);
-    for(double v = vv.low(); v <= vv.high() + 1.e-5;
-        v += (vv.high() - vv.low()) / initial_guesses) {
+  // Try to find a better initial guess by sampling full parameter range
+  const double nGuesses = 10.;
+  const Range<double> uu = parBounds(0);
+  const Range<double> vv = parBounds(1);
+  const double ru = uu.high()-uu.low(), rv = vv.high()-vv.low();
+  const double epsU = 1e-5*ru, epsV = 1e-5*rv;
+  const double du = ru/nGuesses, dv = rv/nGuesses;
+  for(double u = uu.low(); u <= uu.high() + epsU; u += du) {
+    for(double v = vv.low(); v <= vv.high() + epsV; v += dv) {
       GPoint pnt = point(u, v);
       SPoint3 spnt(pnt.x(), pnt.y(), pnt.z());
       double dist = queryPoint.distance(spnt);
-      // fprintf(F,"SP(%g,%g,%g) {%g};\n",pnt.x(), pnt.y(), pnt.z(),dist);
-      // printf("lmocal (%12.5E %12.5E) (point) : %12.5E %12.5E %12.5E (query) : "
-      //        "%12.5E %12.5E %12.5E DSIT %12.5E\n",u,v, pnt.x(), pnt.y(), pnt.z(),
-      //         queryPoint.x(),queryPoint.y(),queryPoint.z(), dist);
       if (dist < min_dist) {
-	// printf("min_dist %f\n", dist);
-	min_dist = dist;
-	min_u = u;
-	min_v = v;
-	// GPoint pnt = point(min_u, min_v);
+        min_dist = dist;
+        min_u = u;
+        min_v = v;
       }
     }
   }
-  // fprintf(F,"};\n");
-  // fclose(F);
-  // getchar();
-  initial_conditions[0] = min_u;
-  initial_conditions[1] = min_v;
-
-  // printf("Initial conditions : %f %f %12.5E\n", min_u, min_v,min_dist);
-  // GPoint pnt = point(min_u, min_v);
-  // printf("Initial conditions (point) : %f %f %f local (%g %g) "
-  //        "Looking for %f %f %f DIST = %12.5E\n",
-  //         pnt.x(), pnt.y(), pnt.z(),min_u,min_v,
-  //         queryPoint.x(),queryPoint.y(),queryPoint.z(), min_dist);
-
-  x.setcontent(dim, &initial_conditions[0]);
 
+  // Set up optimisation problem
+  alglib::ae_int_t dim = 2;
+  alglib::ae_int_t corr = 2; // Num of corrections in the scheme in [3,7]
+  alglib::minlbfgsstate state;
+  alglib::real_1d_array x;
+  const double initialCond[2] = {min_u, min_v};
+  x.setcontent(dim, initialCond);
   minlbfgscreate(2, corr, x, state);
 
-  // Defining the stopping criterion
-  double epsg = 1.e-12;
-  double epsf = 0.0;
-  double epsx = 0.0;
-  alglib::ae_int_t maxits = 500;
-
+  // Set stopping criteria
+  const double epsg = 1.e-12;
+  const double epsf = 0.;
+  const double epsx = 0.;
+  const alglib::ae_int_t maxits = 500;
   minlbfgssetcond(state, epsg, epsf, epsx, maxits);
 
-  // Solving the problem
+  // Solve problem
   data_wrapper w;
   w.set_point(queryPoint);
   w.set_face(this);
-
   minlbfgsoptimize(state, bfgs_callback, NULL, &w);
 
-  // Getting the results
+  // Get results
   alglib::minlbfgsreport rep;
-
   minlbfgsresults(state, x, rep);
-
   GPoint pntF = point(x[0], x[1]);
-  if (rep.terminationtype != 4){
-    // printf("Initial conditions (point) : %f %f %f local (%g %g) "
-    //        "Looking for %f %f %f DIST = %12.5E at the end (%f %f) %f %f %f\n",
-    //         pnt.x(), pnt.y(), pnt.z(),min_u,min_v,
-    //         queryPoint.x(),queryPoint.y(),queryPoint.z(),
-    //         min_dist,x[0],x[1],pntF.x(), pntF.y(), pntF.z());
-    // double DDD =
-    //   ( queryPoint.x() - pntF.x()) * ( queryPoint.x() - pntF.x()) +
-    //   ( queryPoint.y() - pntF.y()) * ( queryPoint.y() - pntF.y()) +
-    //   ( queryPoint.z() - pntF.z()) * ( queryPoint.z() - pntF.z());
-    // if (sqrt(DDD) > 1.e-4)
-    /*
-    printf("Initial conditions (point) : %f %f %f local (%g %g) Looking for %f %f %f "
-           "DIST = %12.5E at the end (%f %f) %f %f %f termination %d\n",
-           pnt.x(), pnt.y(), pnt.z(),min_u,min_v,
-	   queryPoint.x(),queryPoint.y(),queryPoint.z(),
-	   min_dist,x[0],x[1],pntF.x(), pntF.y(), pntF.z(),rep.terminationtype);
-    */
-  }
   return pntF;
+
 #else
   Msg::Error("Closest point not implemented for this type of surface");
   SPoint2 p = parFromPoint(queryPoint, false);
@@ -1420,7 +1408,7 @@ void GFace::addLayersOfQuads(int nLayers, GVertex *gv, double hmin, double ratio
   SVector3 ez (0, 0, 1);
   std::list<GEdgeLoop>::iterator it = edgeLoops.begin();
   FILE *f = Fopen ("coucou.pos","w");
-  fprintf(f,"View \"\"{\n");
+  if(f) fprintf(f,"View \"\"{\n");
   for (; it != edgeLoops.end(); ++it){
     bool found = false;
     // look if this edge loop has the GVertex as an endpoint
@@ -1466,15 +1454,15 @@ void GFace::addLayersOfQuads(int nLayers, GVertex *gv, double hmin, double ratio
 	  double v = p1.y() + n.y() * hmin;
 	  GPoint gp = point(SPoint2(u,v));
 	  additionalVertices.push_back(new MFaceVertex(gp.x(),gp.y(),gp.z(),this,u,v));
-	  fprintf(f,"SP(%g, %g, 0){1};\n",gp.x(),gp.y());
+	  if(f) fprintf(f,"SP(%g, %g, 0){1};\n",gp.x(),gp.y());
 	}
 	hlayer *= ratio;
 	hmin += hlayer;
       }
-      fprintf(f,"};\n");
+      if(f) fprintf(f,"};\n");
     }
   }
-  fclose(f);
+  if(f) fclose(f);
 }
 
 void GFace::relocateMeshVertices()
