@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -58,18 +58,20 @@ class GFace : public GEntity
   // periodic counterparts of edges
   std::map<int,int> edgeCounterparts;
 
+  // encoding of an explicit affine transformation for period meshing
+  std::vector<double> affineTransform;
+
   // an array with additional vertices that are supposed to exist in
   // the final mesh of the model face. This can be used for boundary
   // layer meshes or when using Lloyd-like smoothing algorithms those
   // vertices are classifed on this GFace, their type is MFaceVertex.
   // After mesh generation, those are moved to the mesh_vertices array
+  std::vector<MVertex*> additionalVertices;
 
  public:
   GFace(GModel *model, int tag);
   virtual ~GFace();
 
-  std::vector<MVertex*> additionalVertices;
-
   // delete mesh data
   virtual void deleteMesh();
 
@@ -96,6 +98,10 @@ class GFace : public GEntity
   // edge in the face, not part of any edge loops--use with caution!)
   void delFreeEdge(GEdge *e);
 
+  //find the edge 1 from the list of edges and replace it by edge 2
+  //dont change the edge loops, and is susceptible to break some functionalities
+  void replaceEdge(GEdge *e1, GEdge *e2);
+
   // edge orientations
   virtual std::list<int> orientations() const { return l_dirs; }
 
@@ -115,6 +121,8 @@ class GFace : public GEntity
   // edges that are embedded in the face
   virtual std::list<GVertex*> embeddedVertices() const { return embedded_vertices; }
 
+  std::vector<MVertex*> getEmbeddedMeshVertices() const;
+
   // vertices that bound the face
   virtual std::list<GVertex*> vertices() const;
 
