@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -125,12 +125,6 @@ class GModel
   void _storeVerticesInEntities(std::map<int, MVertex*> &vertices);
   void _storeVerticesInEntities(std::vector<MVertex*> &vertices);
 
-  // remove all mesh vertex associations to geometrical entities and
-  // remove vertices from geometrical entities, then
-  // _associateEntityWithMeshVertices and _storeVerticesInEntities
-  // are called to rebuild the associations
-  void _pruneMeshVertexAssociations();
-
   // store the physical tags in the geometrical entities
   void _storePhysicalTagsInEntities(int dim,
                                     std::map<int, std::map<int, std::string> > &map);
@@ -203,6 +197,11 @@ class GModel
   //delete the mesh stored in entities and call destroMeshCaches
   void deleteMesh();
 
+  // remove all mesh vertex associations to geometrical entities and remove
+  // vertices from geometrical entities, then _associateEntityWithMeshVertices
+  // and _storeVerticesInEntities are called to rebuild the associations
+  void pruneMeshVertexAssociations();
+
   // access internal CAD representations
   GEO_Internals *getGEOInternals(){ return _geo_internals; }
   OCC_Internals *getOCCInternals(){ return _occ_internals; }
@@ -283,7 +282,11 @@ class GModel
   void snapVertices();
 
   // fill a vector containing all the entities in the model
-  void getEntities(std::vector<GEntity*> &entities) const;
+  void getEntities(std::vector<GEntity*> &entities, int dim=-1) const;
+
+  // fill a vector containing all the entities in a given bounding box
+  void getEntitiesInBox(std::vector<GEntity*> &entities, SBoundingBox3d box,
+                        int dim=-1) const;
 
   // return the highest number associated with an elementary entity of
   // a given dimension (or the highest overall if dim < 0)
@@ -293,7 +296,8 @@ class GModel
   bool noPhysicalGroups();
 
   // return all physical groups (one map per dimension: 0-D to 3-D)
-  void getPhysicalGroups(std::map<int, std::vector<GEntity*> > groups[4]);
+  void getPhysicalGroups(std::map<int, std::vector<GEntity*> > groups[4]) const;
+  void getPhysicalGroups(int dim, std::map<int, std::vector<GEntity*> > &groups) const;
 
   // delete physical groups in the model
   void deletePhysicalGroups();
@@ -327,6 +331,12 @@ class GModel
   // "dim" and name "name". return -1 if not found
   int getPhysicalNumber(const int &dim, const std::string & name);
 
+  // set physical tags to entities in a given bounding box
+  void setPhysicalNumToEntitiesInBox(int EntityDimension, int PhysicalNumber,
+                                     std::vector<double> p1, std::vector<double> p2);
+  void setPhysicalNumToEntitiesInBox(int EntityDimension, int PhysicalNumber,
+                                     SBoundingBox3d box);
+
   // get the name (if any) of a given elementary entity of dimension
   // "dim" and id number "num"
   std::string getElementaryName(int dim, int num);
@@ -531,13 +541,9 @@ class GModel
   void    salomeconnect();
   void    occconnect();
 
-	// do stuff for all entities inside a bounding box
-  void    setPeriodicAllFaces(std::vector<double> FaceTranslationVector);
-  void    setPeriodicPairOfFaces(int numFaceMaster, std::vector<int> EdgeListMaster,
-																 int numFaceSlave, std::vector<int> EdgeListSlave);
-  void    setPhysicalNumToEntitiesInBox(int EntityType, int PhysicalGroupNumber,
-																				std::vector<double> p1,std::vector<double> p2);
-
+  void setPeriodicAllFaces(std::vector<double> FaceTranslationVector);
+  void setPeriodicPairOfFaces(int numFaceMaster, std::vector<int> EdgeListMaster,
+                              int numFaceSlave, std::vector<int> EdgeListSlave);
 
   // build a new GModel by cutting the elements crossed by the levelset ls
   // if cutElem is set to false, split the model without cutting the elements
@@ -620,7 +626,7 @@ class GModel
 
   // mesh statistics (saved as a Gmsh post-processing view)
   int writePOS(const std::string &name, bool printElementary,
-               bool printElementNumber, bool printGamma, bool printEta, bool printRho,
+               bool printElementNumber, bool printSICN, bool printGamma, bool printRho,
                bool printDisto, bool saveAll=false, double scalingFactor=1.0);
 
   // Stereo lithography format
