@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -17,8 +17,8 @@
 #include "nodalBasis.h"
 #include "polynomialBasis.h"
 #include "JacobianBasis.h"
+#include "MetricBasis.h"
 #include "GaussIntegration.h"
-
 class GModel;
 
 // A mesh element.
@@ -45,6 +45,10 @@ class MElement
   MElement(int num=0, int part=0);
   virtual ~MElement(){}
 
+  // Create an element from tag
+  static MElement* createElement(int tag, const std::vector<MVertex*>&,
+                                 int num=0, int part=0);
+
   // set/get the tolerance for isInside() test
   static void setTolerance(const double tol){ _isInsideTolerance = tol; }
   static double getTolerance() { return _isInsideTolerance; }
@@ -58,6 +62,15 @@ class MElement
   // return the polynomial order the element
   virtual int getPolynomialOrder() const { return 1; }
 
+  // return true if the element can be considered as a serendipity element
+  virtual bool getIsAssimilatedSerendipity() const {
+    return ElementType::SerendipityFromTag(getTypeForMSH()) > 0;
+  }
+  // return true if the element has to be considered as a serendipity element
+  virtual bool getIsOnlySerendipity() const {
+    return ElementType::SerendipityFromTag(getTypeForMSH()) > 1;
+  }
+
   // get/set the partition to which the element belongs
   virtual int getPartition() const { return _partition; }
   virtual void setPartition(int num){ _partition = (short)num; }
@@ -117,7 +130,7 @@ class MElement
   }
 
   // get the edges
-  virtual int getNumEdges() = 0;
+  virtual int getNumEdges() const = 0;
   virtual MEdge getEdge(int num) const= 0;
 
   // give an MEdge as input and get its local number and sign
@@ -128,21 +141,24 @@ class MElement
 
   // get an edge representation for drawing
   virtual int getNumEdgesRep(bool curved) = 0;
-  virtual void getEdgeRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n) = 0;
+  virtual void getEdgeRep(bool curved, int num, double *x, double *y, double *z,
+                          SVector3 *n) = 0;
 
   // get the faces
   virtual int getNumFaces() = 0;
   virtual MFace getFace(int num) = 0;
 
   // give an MFace as input and get its local number, sign and rotation
-  virtual void getFaceInfo(const MFace & face, int &ithFace, int &sign, int &rot) const
+  virtual void getFaceInfo(const MFace & face, int &ithFace, int &sign,
+                           int &rot) const
   {
     Msg::Error("Face information not available for this element");
   }
 
   // get a face representation for drawing
   virtual int getNumFacesRep(bool curved) = 0;
-  virtual void getFaceRep(bool curved, int num, double *x, double *y, double *z, SVector3 *n) = 0;
+  virtual void getFaceRep(bool curved, int num, double *x, double *y, double *z,
+                          SVector3 *n) = 0;
 
   // get all the vertices on a edge or a face
   virtual void getEdgeVertices(const int num, std::vector<MVertex*> &v) const
@@ -160,6 +176,7 @@ class MElement
   virtual int getNumChildren() const { return 0; }
   virtual MElement *getChild(int i) const { return NULL; }
   virtual bool ownsParent() const { return false; }
+
   // get base element in case of MSubElement
   virtual const MElement *getBaseElement() const { return this; }
   virtual MElement *getBaseElement() { return this; }
@@ -175,22 +192,32 @@ class MElement
   virtual double maxEdge();
   virtual double minEdge();
 
-  // Max. distance between curved and straight element among all high-order nodes
-  double maxDistToStraight();
+  // max. distance between curved and straight element among all high-order nodes
+  double maxDistToStraight() const;
 
   // get the quality measures
   double skewness();
   virtual double rhoShapeMeasure();
   virtual double gammaShapeMeasure(){ return 0.; }
   virtual double etaShapeMeasure(){ return 0.; }
-  virtual double distoShapeMeasure()
+  double distoShapeMeasure()
   {
     double jmin, jmax;
     scaledJacRange(jmin, jmax);
     return jmin;
   }
+  double minSICNShapeMeasure()
+  {
+    double sICNMin, sICNMax;
+    signedInvCondNumRange(sICNMin, sICNMax);
+    return sICNMin;
+  }
   virtual double angleShapeMeasure() { return 1.0; }
-  virtual void scaledJacRange(double &jmin, double &jmax, GEntity *ge = 0);
+  virtual void scaledJacRange(double &jmin, double &jmax, GEntity *ge = 0) const;
+  virtual void idealJacRange(double &jmin, double &jmax, GEntity *ge = 0);
+  virtual void signedInvCondNumRange(double &iCNMin, double &iCNMax, GEntity *ge = 0);
+  virtual double metricShapeMeasure();
+  virtual double metricShapeMeasure2();
 
   // get the radius of the inscribed circle/sphere if it exists,
   // otherwise get the minimum radius of all the circles/spheres
@@ -266,18 +293,22 @@ class MElement
     }
     return detJ;
   }
-  virtual double getPrimaryJacobian(double u, double v, double w, double jac[3][3]);
-  double getJacobianDeterminant(double u, double v, double w)
+  virtual double getPrimaryJacobian(double u, double v, double w, double jac[3][3]) const;
+  double getJacobianDeterminant(double u, double v, double w) const
   {
     double jac[3][3]; return getJacobian(u, v, w, jac);
   }
-  void getSignedJacobian(fullVector<double> &jacobian, int o = -1);
+  void getSignedJacobian(fullVector<double> &jacobian, int o = -1) const;
   void getNodesCoord(fullMatrix<double> &nodesXYZ) const;
-  virtual int getNumShapeFunctions() const{ return getNumVertices(); }
-  virtual int getNumPrimaryShapeFunctions() { return getNumPrimaryVertices(); }
-  virtual const MVertex *getShapeFunctionNode(int i) const{ return getVertex(i); }
+  virtual int getNumShapeFunctions() const { return getNumVertices(); }
+  virtual int getNumPrimaryShapeFunctions() const { return getNumPrimaryVertices(); }
+  virtual const MVertex *getShapeFunctionNode(int i) const { return getVertex(i); }
   virtual MVertex *getShapeFunctionNode(int i) { return getVertex(i); }
 
+  // return the eigenvalues of the metric evaluated at point (u,v,w) in
+  // parametric coordinates
+  virtual double getEigenvaluesMetric(double u, double v, double w, double values[3]) const;
+
   // get the point in cartesian coordinates corresponding to the point
   // (u,v,w) in parametric coordinates
   virtual void pnt(double u, double v, double w, SPoint3 &p) const;
@@ -329,7 +360,7 @@ class MElement
                          int parentNum=0, int dom1Num = 0, int dom2Num = 0,
                          std::vector<short> *ghosts=0);
   virtual void writePOS(FILE *fp, bool printElementary, bool printElementNumber,
-                        bool printGamma, bool printEta, bool printRho,
+                        bool printSICN, bool printGamma, bool printRho,
                         bool printDisto,double scalingFactor=1.0, int elementary=1);
   virtual void writeSTL(FILE *fp, bool binary=false, double scalingFactor=1.0);
   virtual void writeVRML(FILE *fp);
