@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -24,6 +24,7 @@
 #include <FL/Fl_JPEG_Image.H>
 #include <FL/Fl_PNG_Image.H>
 #include <FL/gl.h>
+#include "openglWindow.h"
 #endif
 
 #if defined(HAVE_POPPLER)
@@ -32,8 +33,10 @@
 
 drawContextGlobal *drawContext::_global = 0;
 
-drawContext::drawContext(drawTransform *transform)
-  : _transform(transform)
+extern SPoint2 getGraph2dDataPointForTag(unsigned int);
+
+drawContext::drawContext(openglWindow *window, drawTransform *transform)
+  : _transform(transform), _openglWindow(window)
 {
   // initialize from temp values in global context
   for(int i = 0; i < 3; i++){
@@ -63,6 +66,17 @@ drawContext::~drawContext()
   invalidateQuadricsAndDisplayLists();
 }
 
+bool drawContext::isHighResolution()
+{
+  // this must be dynamic: the high resolution can change when a window is moved
+  // across displays
+#if defined(HAVE_FLTK)
+  return _openglWindow->pixel_w() > _openglWindow->w();
+#else
+  return false;
+#endif
+}
+
 drawContextGlobal *drawContext::global()
 {
   if(!_global) _global = new drawContextGlobal(); // create dummy default
@@ -286,6 +300,7 @@ void drawContext::draw3d()
   drawBackgroundImage(true);
   drawMesh();
   drawPost();
+  drawGraph2d(true);
 }
 
 void drawContext::draw2d()
@@ -296,6 +311,7 @@ void drawContext::draw2d()
 
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
+
   glOrtho((double)viewport[0], (double)viewport[2],
           (double)viewport[1], (double)viewport[3],
           -100., 100.); // in pixels, so we can draw some 3D glyphs
@@ -306,11 +322,10 @@ void drawContext::draw2d()
   glMatrixMode(GL_MODELVIEW);
 
   glLoadIdentity();
-  drawGraph2d();
+  drawGraph2d(false);
   drawText2d();
   if(CTX::instance()->post.draw && !CTX::instance()->stereo)
     drawScales();
-
   if(CTX::instance()->smallAxes)
     drawSmallAxes();
 }
@@ -466,8 +481,8 @@ void drawContext::drawBackgroundImage(bool threeD)
   glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
   glEnable(GL_TEXTURE_2D);
   glBindTexture(GL_TEXTURE_2D, _bgImageTexture);
+  glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
   glBegin(GL_QUADS);
-  glColor4ubv((GLubyte *) & CTX::instance()->color.bg);
   if(threeD){
     glTexCoord2f(1.0f, 1.0f); glVertex2d(x+w, y);
     glTexCoord2f(1.0f, 0.0f); glVertex2d(x+w, y+h);
@@ -740,21 +755,26 @@ void drawContext::initPosition()
 // Takes a cursor position in window coordinates and returns the line (given by
 // a point and a unit direction vector), in real space, that corresponds to that
 // cursor position
-void drawContext::unproject(double x, double y, double p[3], double d[3])
+void drawContext::unproject(double winx, double winy, double p[3], double d[3])
 {
+  if(isHighResolution()){
+    winx *= 2; // true pixels
+    winy *= 2;
+  }
+
   GLint vp[4];
   glGetIntegerv(GL_VIEWPORT, vp);
 
-  y = vp[3] - y;
+  winy = vp[3] - winy;
 
   GLdouble x0, y0, z0, x1, y1, z1;
 
   // we use the stored model and proj matrices instead of directly
   // getGetDouble'ing the matrices since unproject can be called in or after
   // draw2d
-  if(!gluUnProject(x, y, 0.0, model, proj, vp, &x0, &y0, &z0))
+  if(!gluUnProject(winx, winy, 0.0, model, proj, vp, &x0, &y0, &z0))
     Msg::Warning("unproject1 failed");
-  if(!gluUnProject(x, y, 1.0, model, proj, vp, &x1, &y1, &z1))
+  if(!gluUnProject(winx, winy, 1.0, model, proj, vp, &x1, &y1, &z1))
     Msg::Warning("unproject2 failed");
 
   p[0] = x0;
@@ -769,24 +789,24 @@ void drawContext::unproject(double x, double y, double p[3], double d[3])
   d[2] /= len;
 }
 
-void drawContext::viewport2World(double win[3], double xyz[3])
+void drawContext::viewport2World(double vp[3], double xyz[3])
 {
   GLint viewport[4];
   GLdouble model[16], proj[16];
   glGetIntegerv(GL_VIEWPORT, viewport);
   glGetDoublev(GL_PROJECTION_MATRIX, proj);
   glGetDoublev(GL_MODELVIEW_MATRIX, model);
-  gluUnProject(win[0], win[1], win[2], model, proj, viewport, &xyz[0], &xyz[1], &xyz[2]);
+  gluUnProject(vp[0], vp[1], vp[2], model, proj, viewport, &xyz[0], &xyz[1], &xyz[2]);
 }
 
-void drawContext::world2Viewport(double xyz[3], double win[3])
+void drawContext::world2Viewport(double xyz[3], double vp[3])
 {
   GLint viewport[4];
   GLdouble model[16], proj[16];
   glGetIntegerv(GL_VIEWPORT, viewport);
   glGetDoublev(GL_PROJECTION_MATRIX, proj);
   glGetDoublev(GL_MODELVIEW_MATRIX, model);
-  gluProject(xyz[0], xyz[1], xyz[2], model, proj, viewport, &win[0], &win[1], &win[2]);
+  gluProject(xyz[0], xyz[1], xyz[2], model, proj, viewport, &vp[0], &vp[1], &vp[2]);
 }
 
 class hit{
@@ -827,7 +847,8 @@ bool drawContext::select(int type, bool multiple, bool mesh,
                          std::vector<GEdge*> &edges,
                          std::vector<GFace*> &faces,
                          std::vector<GRegion*> &regions,
-                         std::vector<MElement*> &elements)
+                         std::vector<MElement*> &elements,
+                         std::vector<SPoint2> &points)
 {
   vertices.clear();
   edges.clear();
@@ -839,9 +860,9 @@ bool drawContext::select(int type, bool multiple, bool mesh,
   // maximum number of possible hits
   GModel *m = GModel::current();
   int eles = (mesh && CTX::instance()->pickElements) ? 4 * m->getNumMeshElements() : 0;
+  int views = PView::list.size() * 100;
   int size = 7 * (m->getNumVertices() + m->getNumEdges() + m->getNumFaces() +
-                  m->getNumRegions() + eles);
-
+                  m->getNumRegions() + eles) + views;
   if(!size) return false; // the model is empty, don't bother!
 
   // allocate selection buffer
@@ -854,11 +875,28 @@ bool drawContext::select(int type, bool multiple, bool mesh,
   glRenderMode(GL_SELECT);
   glInitNames();
   glPushMatrix();
+
+  // 3d stuff
   initProjection(x, y, w, h);
   initPosition();
   drawGeom();
   if(mesh) drawMesh();
+  drawGraph2d(true);
+
+  // 2d stuff
+  glMatrixMode(GL_PROJECTION);
+  glLoadIdentity();
+  gluPickMatrix((GLdouble)x, (GLdouble)(viewport[3] - y),
+                (GLdouble)w, (GLdouble)h, (GLint *)viewport);
+  glOrtho((double)viewport[0], (double)viewport[2],
+          (double)viewport[1], (double)viewport[3],
+          -100., 100.); // in pixels, so we can draw some 3D glyphs
+  glMatrixMode(GL_MODELVIEW);
+  glLoadIdentity();
+  drawGraph2d(false);
+
   glPopMatrix();
+
   GLint numhits = glRenderMode(GL_RENDER);
   render_mode = drawContext::GMSH_RENDER;
 
@@ -984,12 +1022,20 @@ bool drawContext::select(int type, bool multiple, bool mesh,
           if(!multiple) return true;
         }
         break;
+      case 4:
+        {
+          int tag = hits[i].ient;
+          SPoint2 p = getGraph2dDataPointForTag(tag);
+          points.push_back(p);
+          if(!multiple) return true;
+        }
+        break;
       }
     }
   }
 
   if(vertices.size() || edges.size() || faces.size() ||
-     regions.size() || elements.size())
+     regions.size() || elements.size() || points.size())
     return true;
   return false;
 }
