@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -10,6 +10,7 @@
 #include <vector>
 #include <set>
 #include "SBoundingBox3d.h"
+#include "SPoint2.h"
 #include "Camera.h"
 
 #if defined(WIN32)
@@ -33,6 +34,7 @@ class GEdge;
 class GFace;
 class GRegion;
 class MElement;
+class openglWindow;
 
 class drawTransform {
  public:
@@ -111,6 +113,7 @@ class drawContext {
   std::set<GModel*> _hiddenModels;
   std::set<PView*> _hiddenViews;
   GLuint _bgImageTexture, _bgImageW, _bgImageH;
+  openglWindow *_openglWindow;
  public:
   Camera camera;
   double r[3]; // current Euler angles (in degrees!)
@@ -126,8 +129,9 @@ class drawContext {
   enum RenderMode {GMSH_RENDER=1, GMSH_SELECT=2, GMSH_FEEDBACK=3};
   int render_mode; // current rendering mode
  public:
-  drawContext(drawTransform *transform=0);
+  drawContext(openglWindow *window, drawTransform *transform=0);
   ~drawContext();
+  bool isHighResolution();
   void copyViewAttributes(drawContext *other)
   {
     camera = other->camera;
@@ -187,13 +191,13 @@ class drawContext {
   void initProjection(int xpick=0, int ypick=0, int wpick=0, int hpick=0);
   void initRenderModel();
   void initPosition();
-  void unproject(double x, double y, double p[3], double d[3]);
-  void viewport2World(double win[3], double xyz[3]);
-  void world2Viewport(double xyz[3], double win[3]);
+  void unproject(double winx, double winy, double p[3], double d[3]);
+  void viewport2World(double vp[3], double xyz[3]);
+  void world2Viewport(double xyz[3], double vp[3]);
   bool select(int type, bool multiple, bool mesh, int x, int y, int w, int h,
               std::vector<GVertex*> &vertices, std::vector<GEdge*> &edges,
               std::vector<GFace*> &faces, std::vector<GRegion*> &regions,
-              std::vector<MElement*> &elements);
+              std::vector<MElement*> &elements, std::vector<SPoint2> &points);
   int fix2dCoordinates(double *x, double *y);
   void draw3d();
   void draw2d();
@@ -203,7 +207,7 @@ class drawContext {
   void drawBackgroundGradient();
   void drawBackgroundImage(bool moving);
   void drawText2d();
-  void drawGraph2d();
+  void drawGraph2d(bool inModelCoordinates);
   void drawAxis(double xmin, double ymin, double zmin,
                 double xmax, double ymax, double zmax,
                 int nticks, int mikado);
