@@ -1,8 +1,9 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
 
+#include <algorithm>
 #include "drawContext.h"
 #include "PView.h"
 #include "PViewOptions.h"
@@ -201,7 +202,7 @@ static bool getGraphData(PView *p, std::vector<double> &x, double &xmin,
 
 static void drawGraphAxes(drawContext *ctx, PView *p, double xleft, double ytop,
                           double width, double height, double xmin, double xmax,
-                          int tic, int overlay)
+                          double tic, int overlay, bool inModelCoordinates)
 {
   PViewData *data = p->getData();
   PViewOptions *opt = p->getOptions();
@@ -212,7 +213,7 @@ static void drawGraphAxes(drawContext *ctx, PView *p, double xleft, double ytop,
 
   if(width <= 0 || height <= 0) return;
 
-  if(!overlay){
+  if(!overlay && !inModelCoordinates){
     int alpha = CTX::instance()->unpackAlpha(opt->color.background2d);
     if(alpha != 0){
       glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
@@ -234,7 +235,17 @@ static void drawGraphAxes(drawContext *ctx, PView *p, double xleft, double ytop,
   // height above ref. point
   double font_a = font_h - drawContext::global()->getStringDescent();
 
-  glPointSize((float)CTX::instance()->pointSize);
+  if(inModelCoordinates){
+    double ss = ctx->pixel_equiv_x / ctx->s[0];
+    font_h *= ss;
+    font_a *= ss;
+  }
+
+  double ps = CTX::instance()->pointSize;
+  if(ctx->isHighResolution())
+    ps *= CTX::instance()->highResolutionPointSizeFactor;
+
+  glPointSize((float)ps);
   gl2psPointSize((float)(CTX::instance()->pointSize *
                          CTX::instance()->print.epsPointSizeFactor));
 
@@ -339,8 +350,9 @@ static void drawGraphAxes(drawContext *ctx, PView *p, double xleft, double ytop,
     int nb = opt->axesTics[0];
     char tmp[256];
     sprintf(tmp, opt->axesFormat[0].c_str(), -M_PI * 1.e4);
-    if((nb - 1) * drawContext::global()->getStringWidth(tmp) > width)
-      nb = (int)(width / drawContext::global()->getStringWidth(tmp)) + 1;
+    double ww = drawContext::global()->getStringWidth(tmp);
+    if(inModelCoordinates) ww *= ctx->pixel_equiv_x / ctx->s[0];
+    if((nb - 1) * ww > width) nb = (int)(width / ww) + 1;
     if(nb == 1) nb++;
 
     double dx = width / (double)(nb - 1);
@@ -385,10 +397,28 @@ static void drawGraphAxes(drawContext *ctx, PView *p, double xleft, double ytop,
 
 }
 
+static std::map<SPoint2, unsigned int> tags;
+static std::map<unsigned int, SPoint2> tags_rev;
+
+static unsigned int getTagForGraph2dDataPoint(SPoint2 p)
+{
+  std::map<SPoint2, unsigned int>::iterator it = tags.find(p);
+  if(it != tags.end()) return it->second;
+  int t = tags.size();
+  tags[p] = t;
+  tags_rev[t] = p;
+  return t;
+}
+
+SPoint2 getGraph2dDataPointForTag(unsigned int tag)
+{
+  return tags_rev[tag];
+}
+
 static void addGraphPoint(drawContext *ctx, PView *p, double xleft, double ytop,
                           double width, double height, double x, double y,
                           double xmin, double xmax, double ymin, double ymax,
-                          bool numeric, bool sphere)
+                          bool numeric, bool singlePoint, bool inModelCoordinates)
 {
   PViewOptions *opt = p->getOptions();
 
@@ -409,29 +439,58 @@ static void addGraphPoint(drawContext *ctx, PView *p, double xleft, double ytop,
   if(y >= ymin && y <= ymax){
     unsigned int col = opt->getColor(y, ymin, ymax, true);
     glColor4ubv((GLubyte *) &col);
+
+    if(singlePoint && ctx->render_mode == drawContext::GMSH_SELECT){
+      glPushName(4);
+      glPushName(getTagForGraph2dDataPoint(SPoint2(x, y)));
+    }
+
     if(numeric){
-      glRasterPos2d(px + 3, py + 3);
+      double offset = 3;
+      if(inModelCoordinates) offset *= ctx->pixel_equiv_x / ctx->s[0];
+      glRasterPos2d(px + offset, py + offset);
       char label[256];
       sprintf(label, opt->format.c_str(), y);
       ctx->drawString(label);
     }
-    else if(sphere)
-      ctx->drawSphere(opt->pointSize, px, py, 0, 10, 10, opt->light);
-    else
+    else if(singlePoint && (opt->pointType == 1 || opt->pointType == 3)){
+      double ps = CTX::instance()->pointSize;
+      if(ctx->isHighResolution())
+        ps *= CTX::instance()->highResolutionPointSizeFactor;
+      if(inModelCoordinates)
+        ctx->drawSphere(ps, px, py, 0, opt->light);
+      else
+        ctx->drawSphere(ps, px, py, 0, 10, 10, opt->light);
+    }
+    else{
+      if(singlePoint) glBegin(GL_POINTS);
       glVertex2d(px, py);
+      if(singlePoint) glEnd();
+    }
+
+    if(singlePoint && ctx->render_mode == drawContext::GMSH_SELECT){
+      glPopName();
+      glPopName();
+    }
+
   }
 }
 
 static void drawGraphCurves(drawContext *ctx, PView *p, double xleft, double ytop,
                             double width, double height, std::vector<double> &x,
                             double xmin, double xmax,
-                            std::vector<std::vector<double> > &y)
+                            std::vector<std::vector<double> > &y,
+                            bool inModelCoordinates)
 {
   if(width <= 0 || height <= 0) return;
 
   PViewOptions *opt = p->getOptions();
 
-  glPointSize((float)opt->pointSize);
+  double ps = CTX::instance()->pointSize;
+  if(ctx->isHighResolution())
+    ps *= CTX::instance()->highResolutionPointSizeFactor;
+
+  glPointSize((float)ps);
   gl2psPointSize((float)(opt->pointSize * CTX::instance()->print.epsPointSizeFactor));
 
   glLineWidth((float)opt->lineWidth);
@@ -448,7 +507,8 @@ static void drawGraphCurves(drawContext *ctx, PView *p, double xleft, double yto
       glBegin(GL_LINE_STRIP);
       for(unsigned int j = 0; j < x.size(); j++)
         addGraphPoint(ctx, p, xleft, ytop, width, height, x[j], y[i][j],
-                      xmin, xmax, opt->tmpMin, opt->tmpMax, false, false);
+                      xmin, xmax, opt->tmpMin, opt->tmpMax, false, false,
+                      inModelCoordinates);
       glEnd();
       if(opt->useStipple){
         glDisable(GL_LINE_STIPPLE);
@@ -460,25 +520,25 @@ static void drawGraphCurves(drawContext *ctx, PView *p, double xleft, double yto
   if(opt->intervalsType == PViewOptions::Iso ||
      opt->intervalsType == PViewOptions::Discrete ||
      opt->intervalsType == PViewOptions::Numeric){
-    bool sphere =  (opt->pointType == 1 || opt->pointType == 3);
-    if(!sphere) glBegin(GL_POINTS);
     for(unsigned int i = 0; i < y.size(); i++)
       for(unsigned int j = 0; j < x.size(); j++)
         addGraphPoint(ctx, p, xleft, ytop, width, height, x[j], y[i][j],
-                      xmin, xmax, opt->tmpMin, opt->tmpMax, false, sphere);
-    if(!sphere) glEnd();
+                      xmin, xmax, opt->tmpMin, opt->tmpMax, false, true,
+                      inModelCoordinates);
   }
 
   if(opt->intervalsType == PViewOptions::Numeric){
     for(unsigned int i = 0; i < y.size(); i++)
       for(unsigned int j = 0; j < x.size(); j++)
         addGraphPoint(ctx, p, xleft, ytop, width, height, x[j], y[i][j],
-                      xmin, xmax, opt->tmpMin, opt->tmpMax, true, false);
+                      xmin, xmax, opt->tmpMin, opt->tmpMax, true, true,
+                      inModelCoordinates);
   }
 }
 
 static void drawGraph(drawContext *ctx, PView *p, double xleft, double ytop,
-                      double width, double height, int tic, int overlay=0)
+                      double width, double height, double tic, int overlay=0,
+                      bool inModelCoordinates=false)
 {
   std::vector<double> x;
   std::vector<std::vector<double> > y;
@@ -511,11 +571,13 @@ static void drawGraph(drawContext *ctx, PView *p, double xleft, double ytop,
     opt->tmpMax = log10(opt->tmpMax);
   }
 
-  drawGraphAxes(ctx, p, xleft, ytop, width, height, xmin, xmax, tic, overlay);
-  drawGraphCurves(ctx, p, xleft, ytop, width, height, x, xmin, xmax, y);
+  drawGraphAxes(ctx, p, xleft, ytop, width, height, xmin, xmax, tic, overlay,
+                inModelCoordinates);
+  drawGraphCurves(ctx, p, xleft, ytop, width, height, x, xmin, xmax, y,
+                  inModelCoordinates);
 }
 
-void drawContext::drawGraph2d()
+void drawContext::drawGraph2d(bool inModelCoordinates)
 {
   std::vector<PView*> graphs;
   for(unsigned int i = 0; i < PView::list.size(); i++){
@@ -529,8 +591,8 @@ void drawContext::drawGraph2d()
 
   drawContext::global()->setFont(CTX::instance()->glFontEnum,
                                  CTX::instance()->glFontSize);
-  const int tic = 5; // size of tic marks and interline
-  const int mx = 25, my = 5; // x- and y-margin
+  double tic = 5; // size of tic marks and interline
+  double mx = 25, my = 5; // x- and y-margin
   double xsep = 0., ysep = drawContext::global()->getStringHeight() + tic;
   char label[1024];
   for(unsigned int i = 0; i < graphs.size(); i++){
@@ -540,6 +602,15 @@ void drawContext::drawGraph2d()
   }
   xsep += tic;
 
+  if(inModelCoordinates){
+    double ss = pixel_equiv_x / s[0];
+    tic *= ss;
+    mx *= ss;
+    my *= ss;
+    xsep *= ss;
+    ysep *= ss;
+  }
+
   //  +------------------winw-------------------+
   //  |       my+3*ysep                         |
   //  |mx+xsep+---w---+mx+2*xsep+---w---+mx+xsep|
@@ -564,7 +635,15 @@ void drawContext::drawGraph2d()
     double y = viewport[1] + my + 3 * ysep;
     PView *p = graphs[i];
     PViewOptions *opt = graphs[i]->getOptions();
-    if(opt->autoPosition == 1){ // automatic
+    if(opt->autoPosition == 0 && !inModelCoordinates){ // manual
+      double x = opt->position[0], y = opt->position[1];
+      int center = fix2dCoordinates(&x, &y);
+      drawGraph(this, p,
+                x - (center & 1 ? opt->size[0] / 2. : 0),
+                y + (center & 2 ? opt->size[1] / 2. : 0),
+                opt->size[0], opt->size[1], tic);
+    }
+    else if(opt->autoPosition == 1 && !inModelCoordinates){ // automatic
       if(graphs.size() == 1){
         double w = winw - 2 * mx - 2 * xsep;
         double h = winh - 2 * my - 7 * ysep;
@@ -584,7 +663,7 @@ void drawContext::drawGraph2d()
         drawGraph(this, p, x, viewport[3] - y, w, h, tic);
       }
     }
-    else if(opt->autoPosition >= 2 && opt->autoPosition <= 11){
+    else if(opt->autoPosition >= 2 && opt->autoPosition <= 11 && !inModelCoordinates){
       // top left (2), top right (3), bottom left (4), bottom right (5), top
       // half (6), bottom half (7), left half (8), right half (9), full (10),
       // top third (11)
@@ -607,13 +686,9 @@ void drawContext::drawGraph2d()
       if(opt->axes)
         overlay[opt->autoPosition] += (opt->axesLabel[0].size() ? 2 : 1);
     }
-    else{ // manual
-      double x = opt->position[0], y = opt->position[1];
-      int center = fix2dCoordinates(&x, &y);
-      drawGraph(this, p,
-                x - (center & 1 ? opt->size[0] / 2. : 0),
-                y + (center & 2 ? opt->size[1] / 2. : 0),
-                opt->size[0], opt->size[1], tic);
+    else if(opt->autoPosition == 12 && inModelCoordinates){ // in model coordinates
+      drawGraph(this, p, opt->position[0], opt->position[1] + opt->size[1],
+                opt->size[0], opt->size[1], tic, 0, true);
     }
   }
 }
