@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -8,7 +8,7 @@
 #include "meshGEdge.h"
 #include "GEdge.h"
 #include "MLine.h"
-#include "BackgroundMesh.h"
+#include "BackgroundMeshTools.h"
 #include "boundaryLayersData.h"
 #include "Numeric.h"
 #include "GmshMessage.h"
@@ -318,7 +318,8 @@ static void printFandPrimitive(int tag, std::vector<IntPoint> &Points)
 */
 
 // new algo for recombining + splitting
-static int increaseN (int N){
+static int increaseN (int N)
+{
   if (((N+1)/2 - 1) % 2 != 0) return N+2;
   return N;
 }
