@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -58,7 +58,7 @@ static void computeElementShapes(GFace *gf, double &worst, double &avg,
   nT = 0;
   greaterThan = 0;
   for(unsigned int i = 0; i < gf->triangles.size(); i++){
-    double q = qmTriangle(gf->triangles[i], QMTRI_RHO);
+    double q = qmTriangle::gamma(gf->triangles[i]);
     if(q > .9) greaterThan++;
     avg += q;
     worst = std::min(worst, q);
@@ -94,7 +94,10 @@ public:
 	  std::vector<MLine*> temp;
 	  (*ite)->mesh_vertices.clear();
 	  for(unsigned int i = 0; i< (*ite)->lines.size(); i+=2){
-	    if (i+1 >= (*ite)->lines.size())Msg::Fatal("1D mesh cannot be divided by 2");
+	    if (i+1 >= (*ite)->lines.size()){
+	      Msg::Error("1D mesh cannot be divided by 2");
+	      break;
+	    }
 	    MVertex *v1 = (*ite)->lines[i]->getVertex(0);
 	    MVertex *v2 = (*ite)->lines[i]->getVertex(1);
 	    MVertex *v3 = (*ite)->lines[i+1]->getVertex(1);
@@ -156,7 +159,7 @@ public:
 	MEdge E = _gf->quadrangles[i]->getEdge(j);
 	SPoint2 p1, p2;
 	reparamMeshEdgeOnFace(E.getVertex(0),E.getVertex(1),_gf,p1,p2);
-	std::map<MEdge, MVertex *>::iterator it = _middle.find(E);
+	std::map<MEdge, MVertex *, Less_Edge>::iterator it = _middle.find(E);
 	std::map<MEdge, MVertex *, Less_Edge>::iterator it2 = eds.find(E);
 	m[j] = p1;
 	if (it == _middle.end() && it2 == eds.end()){
@@ -328,116 +331,137 @@ static void copyMesh(GFace *source, GFace *target)
   }
 
   bool translation = true;
-  bool rotation = false;
-
   SVector3 DX;
-  int count = 0;
-  for (std::map<MVertex*, MVertex*>::iterator it = vs2vt.begin();
-       it != vs2vt.end() ; ++it){
-    MVertex *vs = it->first;
-    MVertex *vt = it->second;
-    if (count == 0)
-      DX = SVector3(vt->x() - vs->x(), vt->y() - vs->y(), vt->z() - vs->z());
-    else {
-      SVector3 DX2 = DX - SVector3 (vt->x() - vs->x(), vt->y() - vs->y(),
-                                    vt->z() - vs->z());
-      if (DX2.norm() > DX.norm() * 1.e-5) translation = false;
-    }
-    count ++;
-  }
 
+  bool rotation = false;
   double rot[3][3] ;
   myLine LINE;
   double ANGLE=0;
-  if (!translation){
-    count = 0;
-    rotation = true;
-    std::vector<SPoint3> mps, mpt;
-    for (std::map<MVertex*, MVertex*>::iterator it = vs2vt.begin();
-         it != vs2vt.end() ; ++it){
-      MVertex *vs = it->first;
-      MVertex *vt = it->second;
-      mps.push_back(SPoint3(vs->x(), vs->y(), vs->z()));
-      mpt.push_back(SPoint3(vt->x(), vt->y(), vt->z()));
-    }
-    mean_plane mean_source, mean_target;
-    computeMeanPlaneSimple(mps, mean_source);
-    computeMeanPlaneSimple(mpt, mean_target);
-    myPlane PLANE_SOURCE(SPoint3(mean_source.x,mean_source.y,mean_source.z),
-                         SVector3(mean_source.a,mean_source.b,mean_source.c));
-    myPlane PLANE_TARGET(SPoint3(mean_target.x,mean_target.y,mean_target.z),
-                         SVector3(mean_target.a,mean_target.b,mean_target.c));
-    LINE = myLine(PLANE_SOURCE, PLANE_TARGET);
 
-    // LINE is the axis of rotation
-    // let us compute the angle of rotation
-    count = 0;
-    for (std::map<MVertex*, MVertex*>::iterator it = vs2vt.begin();
-         it != vs2vt.end(); ++it){
-      MVertex *vs = it->first;
-      MVertex *vt = it->second;
-      // project both points on the axis: that should be the same point !
-      SPoint3 ps = SPoint3(vs->x(), vs->y(), vs->z());
-      SPoint3 pt = SPoint3(vt->x(), vt->y(), vt->z());
-      SPoint3 p_ps = LINE.orthogonalProjection(ps);
-      SPoint3 p_pt = LINE.orthogonalProjection(pt);
-      SVector3 dist1 = ps - pt;
-      SVector3 dist2 = p_ps - p_pt;
-      if (dist1.norm() > CTX::instance()->geom.tolerance){
-        if (dist2.norm() > 1.e-8 * dist1.norm()){
-          rotation = false;
-        }
-        SVector3 t1 = ps - p_ps;
-        SVector3 t2 = pt - p_pt;
-        if (t1.norm() > 1.e-8 * dist1.norm()){
-          if (count == 0)
-            ANGLE = myAngle(t1, t2, LINE.t);
-          else {
-            double ANGLE2 = myAngle(t1, t2, LINE.t);
-            if (fabs (ANGLE2 - ANGLE) > 1.e-8){
-              rotation = false;
-            }
-          }
-          count++;
-        }
-      }
-    }
+  bool affine = false;
+  double mat[4][4];
 
-    if (rotation){
-      Msg::Info("Periodic mesh rotation found: axis (%g,%g,%g) point (%g %g %g) angle %g",
-                LINE.t.x(), LINE.t.y(), LINE.t.z(), LINE.p.x(), LINE.p.y(), LINE.p.z(),
-                ANGLE * 180 / M_PI);
-      double ux = LINE.t.x();
-      double uy = LINE.t.y();
-      double uz = LINE.t.z();
-      rot[0][0] = cos (ANGLE) + ux*ux*(1.-cos(ANGLE));
-      rot[0][1] = ux*uy*(1.-cos(ANGLE)) - uz * sin(ANGLE);
-      rot[0][2] = ux*uz*(1.-cos(ANGLE)) + uy * sin(ANGLE);
-      rot[1][0] = ux*uy*(1.-cos(ANGLE)) + uz * sin(ANGLE);
-      rot[1][1] = cos (ANGLE) + uy*uy*(1.-cos(ANGLE));
-      rot[1][2] = uy*uz*(1.-cos(ANGLE)) - ux * sin(ANGLE);
-      rot[2][0] = ux*uz*(1.-cos(ANGLE)) - uy * sin(ANGLE);
-      rot[2][1] = uy*uz*(1.-cos(ANGLE)) + ux * sin(ANGLE);
-      rot[2][2] = cos (ANGLE) + uz*uz*(1.-cos(ANGLE));
-    }
-    else {
-      Msg::Error("Only rotations or translations can be currently taken into account "
-                 "for periodic faces: face %d not meshed", target->tag());
-      return;
-    }
+  if(target->affineTransform.size() == 16){
+    Msg::Info("Affine transformation specified");
+    affine = true;
+    for(int i = 0; i < 4; i++)
+      for(int j = 0; j < 4; j++)
+    	mat[i][j] = target->affineTransform[4 * i + j];
   }
   else{
-    Msg::Info("Periodic mesh translation found: dx = (%g,%g,%g)",
-              DX.x(), DX.y(), DX.z());
+    int count = 0;
+    for (std::map<MVertex*, MVertex*>::iterator it = vs2vt.begin();
+	 it != vs2vt.end() ; ++it){
+      MVertex *vs = it->first;
+      MVertex *vt = it->second;
+      if (count == 0)
+	DX = SVector3(vt->x() - vs->x(), vt->y() - vs->y(), vt->z() - vs->z());
+      else {
+	SVector3 DX2 = DX - SVector3(vt->x() - vs->x(), vt->y() - vs->y(),
+				     vt->z() - vs->z());
+	if (DX2.norm() > DX.norm() * 1.e-5) translation = false;
+      }
+      count ++;
+    }
+
+    if (!translation){
+      count = 0;
+      rotation = true;
+      std::vector<SPoint3> mps, mpt;
+      for (std::map<MVertex*, MVertex*>::iterator it = vs2vt.begin();
+	   it != vs2vt.end() ; ++it){
+	MVertex *vs = it->first;
+	MVertex *vt = it->second;
+	mps.push_back(SPoint3(vs->x(), vs->y(), vs->z()));
+	mpt.push_back(SPoint3(vt->x(), vt->y(), vt->z()));
+      }
+      mean_plane mean_source, mean_target;
+      computeMeanPlaneSimple(mps, mean_source);
+      computeMeanPlaneSimple(mpt, mean_target);
+      myPlane PLANE_SOURCE(SPoint3(mean_source.x,mean_source.y,mean_source.z),
+			   SVector3(mean_source.a,mean_source.b,mean_source.c));
+      myPlane PLANE_TARGET(SPoint3(mean_target.x,mean_target.y,mean_target.z),
+			   SVector3(mean_target.a,mean_target.b,mean_target.c));
+      LINE = myLine(PLANE_SOURCE, PLANE_TARGET);
+
+      // LINE is the axis of rotation
+      // let us compute the angle of rotation
+      count = 0;
+      for (std::map<MVertex*, MVertex*>::iterator it = vs2vt.begin();
+	   it != vs2vt.end(); ++it){
+	MVertex *vs = it->first;
+	MVertex *vt = it->second;
+	// project both points on the axis: that should be the same point !
+	SPoint3 ps = SPoint3(vs->x(), vs->y(), vs->z());
+	SPoint3 pt = SPoint3(vt->x(), vt->y(), vt->z());
+	SPoint3 p_ps = LINE.orthogonalProjection(ps);
+	SPoint3 p_pt = LINE.orthogonalProjection(pt);
+	SVector3 dist1 = ps - pt;
+	SVector3 dist2 = p_ps - p_pt;
+	if (dist1.norm() > CTX::instance()->geom.tolerance){
+	  if (dist2.norm() > 1.e-8 * dist1.norm()){
+	    rotation = false;
+	  }
+	  SVector3 t1 = ps - p_ps;
+	  SVector3 t2 = pt - p_pt;
+	  if (t1.norm() > 1.e-8 * dist1.norm()){
+	    if (count == 0)
+	      ANGLE = myAngle(t1, t2, LINE.t);
+	    else {
+	      double ANGLE2 = myAngle(t1, t2, LINE.t);
+	      if (fabs (ANGLE2 - ANGLE) > 1.e-8){
+		rotation = false;
+	      }
+	    }
+	    count++;
+	  }
+	}
+      }
+
+      if (rotation){
+	Msg::Info("Periodic mesh rotation found: axis (%g,%g,%g) point (%g %g %g) angle %g",
+		  LINE.t.x(), LINE.t.y(), LINE.t.z(), LINE.p.x(), LINE.p.y(), LINE.p.z(),
+		  ANGLE * 180 / M_PI);
+	double ux = LINE.t.x();
+	double uy = LINE.t.y();
+	double uz = LINE.t.z();
+	rot[0][0] = cos (ANGLE) + ux*ux*(1.-cos(ANGLE));
+	rot[0][1] = ux*uy*(1.-cos(ANGLE)) - uz * sin(ANGLE);
+	rot[0][2] = ux*uz*(1.-cos(ANGLE)) + uy * sin(ANGLE);
+	rot[1][0] = ux*uy*(1.-cos(ANGLE)) + uz * sin(ANGLE);
+	rot[1][1] = cos (ANGLE) + uy*uy*(1.-cos(ANGLE));
+	rot[1][2] = uy*uz*(1.-cos(ANGLE)) - ux * sin(ANGLE);
+	rot[2][0] = ux*uz*(1.-cos(ANGLE)) - uy * sin(ANGLE);
+	rot[2][1] = uy*uz*(1.-cos(ANGLE)) + ux * sin(ANGLE);
+	rot[2][2] = cos (ANGLE) + uz*uz*(1.-cos(ANGLE));
+      }
+      else {
+	Msg::Error("Only rotations or translations can be currently taken into account "
+		   "automatically for periodic faces: face %d not meshed", target->tag());
+	return;
+      }
+    }
+    else{
+      Msg::Info("Periodic mesh translation found: dx = (%g,%g,%g)",
+		DX.x(), DX.y(), DX.z());
+    }
   }
 
-  // now transform !!!
+  // now transform
   for(unsigned int i = 0; i < source->mesh_vertices.size(); i++){
     MVertex *vs = source->mesh_vertices[i];
-
     SPoint2 XXX;
-    if (translation) {
-      SPoint3 tp (vs->x() + DX.x(),vs->y() + DX.y(),vs->z() + DX.z());
+    if (affine) {
+      double ps[4] = {vs->x(), vs->y(), vs->z(), 1.};
+      double res[4] = {0., 0., 0., 0.};
+      for(int i = 0; i < 4; i++)
+	for(int j = 0; j < 4; j++)
+	  res[i] += mat[i][j] * ps[j];
+      SPoint3 tp (res[0], res[1], res[2]);
+      XXX = target->parFromPoint(tp);
+    }
+    else if (translation) {
+      SPoint3 tp (vs->x() + DX.x(), vs->y() + DX.y(), vs->z() + DX.z());
       XXX = target->parFromPoint(tp);
     }
     else if (rotation){
@@ -462,10 +486,11 @@ static void copyMesh(GFace *source, GFace *target)
       vt[j] = vs2vt[vs];
     }
     if (!vt[0] || !vt[1] ||!vt[2]){
-      Msg::Fatal("Yet another error in the copyMesh procedure %p %p %p %d %d %d",
+      Msg::Error("Problem in mesh copying procedure %p %p %p %d %d %d",
 		 vt[0], vt[1], vt[2], source->triangles[i]->getVertex(0)->onWhat()->dim(),
 		 source->triangles[i]->getVertex(1)->onWhat()->dim(),
 		 source->triangles[i]->getVertex(2)->onWhat()->dim());
+      return;
     }
     target->triangles.push_back(new MTriangle(vt[0], vt[1], vt[2]));
   }
@@ -476,7 +501,7 @@ static void copyMesh(GFace *source, GFace *target)
     MVertex *v3 = vs2vt[source->quadrangles[i]->getVertex(2)];
     MVertex *v4 = vs2vt[source->quadrangles[i]->getVertex(3)];
     if (!v1 || !v2 || !v3 || !v4){
-      Msg::Fatal("Yet another error in the copymesh procedure %p %p %p %p %d %d %d %d",
+      Msg::Error("Problem in mesh copying procedure %p %p %p %p %d %d %d %d",
 		 v1, v2, v3, v4,
 		 source->quadrangles[i]->getVertex(0)->onWhat()->dim(),
 		 source->quadrangles[i]->getVertex(1)->onWhat()->dim(),
@@ -754,7 +779,7 @@ static void modifyInitialMeshForTakingIntoAccountBoundaryLayers(GFace *gf)
   edges.insert(edges.begin(), embedded_edges.begin(),embedded_edges.end());
   std::list<GEdge*>::iterator ite = edges.begin();
   FILE *ff2 = Fopen ("tato.pos","w");
-  fprintf(ff2,"View \" \"{\n");
+  if(ff2) fprintf(ff2,"View \" \"{\n");
   std::set<MVertex*> verts;
   while(ite != edges.end()){
     for(unsigned int i = 0; i< (*ite)->lines.size(); i++){
@@ -787,13 +812,14 @@ static void modifyInitialMeshForTakingIntoAccountBoundaryLayers(GFace *gf)
 	  MQuadrangle *qq = new MQuadrangle(v11,v21,v22,v12);
 	  myCol.push_back(qq);
 	  blQuads.push_back(qq);
-	  fprintf(ff2,"SQ (%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g){1,1,1,1};\n",
-		  v11->x(),v11->y(),v11->z(),
-		  v12->x(),v12->y(),v12->z(),
-		  v22->x(),v22->y(),v22->z(),
-		  v21->x(),v21->y(),v21->z());
+          if(ff2)
+            fprintf(ff2,"SQ (%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g){1,1,1,1};\n",
+                    v11->x(),v11->y(),v11->z(),
+                    v12->x(),v12->y(),v12->z(),
+                    v22->x(),v22->y(),v22->z(),
+                    v21->x(),v21->y(),v21->z());
 	}
-	//	int M = std::max(c1._column.size(),c2._column.size());
+	// int M = std::max(c1._column.size(),c2._column.size());
 	for (unsigned int l=0;l<myCol.size();l++)_columns->_toFirst[myCol[l]] = myCol[0];
 	_columns->_elemColumns[myCol[0]] = myCol;
       }
@@ -827,20 +853,22 @@ static void modifyInitialMeshForTakingIntoAccountBoundaryLayers(GFace *gf)
 	  MQuadrangle *qq = new MQuadrangle(v11,v12,v22,v21);
 	  myCol.push_back(qq);
 	  blQuads.push_back(qq);
-	  fprintf(ff2,"SQ (%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g){1,1,1,1};\n",
-		  v11->x(),v11->y(),v11->z(),
-		  v12->x(),v12->y(),v12->z(),
-		  v22->x(),v22->y(),v22->z(),
-		  v21->x(),v21->y(),v21->z());
+	  if(ff2)
+            fprintf(ff2,"SQ (%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g,%g){1,1,1,1};\n",
+                    v11->x(),v11->y(),v11->z(),
+                    v12->x(),v12->y(),v12->z(),
+                    v22->x(),v22->y(),v22->z(),
+                    v21->x(),v21->y(),v21->z());
 	}
 	else {
 	  MTriangle *qq = new MTriangle(v,v22,v21);
 	  myCol.push_back(qq);
 	  blTris.push_back(qq);
-	  fprintf(ff2,"ST (%g,%g,%g,%g,%g,%g,%g,%g,%g){1,1,1,1};\n",
-		  v->x(),v->y(),v->z(),
-		  v22->x(),v22->y(),v22->z(),
-		  v21->x(),v21->y(),v21->z());
+          if(ff2)
+            fprintf(ff2,"ST (%g,%g,%g,%g,%g,%g,%g,%g,%g){1,1,1,1};\n",
+                    v->x(),v->y(),v->z(),
+                    v22->x(),v22->y(),v22->z(),
+                    v21->x(),v21->y(),v21->z());
 	}
       }
     }
@@ -848,8 +876,10 @@ static void modifyInitialMeshForTakingIntoAccountBoundaryLayers(GFace *gf)
     _columns->_elemColumns[myCol[0]] = myCol;
   }
 
-  fprintf(ff2,"};\n");
-  fclose(ff2);
+  if(ff2){
+    fprintf(ff2,"};\n");
+    fclose(ff2);
+  }
 
   //  filterOverlappingElements (blTris,blQuads,_columns->_elemColumns,_columns->_toFirst);
 
@@ -875,16 +905,18 @@ static void modifyInitialMeshForTakingIntoAccountBoundaryLayers(GFace *gf)
   std::set<MEdge,Less_Edge>::iterator it =  bedges.begin();
 
   FILE *ff = Fopen ("toto.pos","w");
-  fprintf(ff,"View \" \"{\n");
+  if(ff) fprintf(ff,"View \" \"{\n");
   for (; it != bedges.end(); ++it){
     ne.lines.push_back(new MLine (it->getVertex(0),it->getVertex(1)));
-    fprintf(ff,"SL (%g,%g,%g,%g,%g,%g){1,1};\n",
-	    it->getVertex(0)->x(),it->getVertex(0)->y(),it->getVertex(0)->z(),
-	    it->getVertex(1)->x(),it->getVertex(1)->y(),it->getVertex(1)->z());
+    if(ff)
+      fprintf(ff,"SL (%g,%g,%g,%g,%g,%g){1,1};\n",
+              it->getVertex(0)->x(),it->getVertex(0)->y(),it->getVertex(0)->z(),
+              it->getVertex(1)->x(),it->getVertex(1)->y(),it->getVertex(1)->z());
+  }
+  if(ff){
+    fprintf(ff,"};\n");
+    fclose(ff);
   }
-  fprintf(ff,"};\n");
-  fclose(ff);
-
   hop.push_back(&ne);
 
   deMeshGFace kil_;
@@ -1077,7 +1109,7 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
   }
 
   // build a set with all points of the boundaries
-  std::set<MVertex*> all_vertices, boundary;
+  std::set<MVertex*, MVertexLessThanNum> all_vertices, boundary;
   std::list<GEdge*>::iterator ite = edges.begin();
   while(ite != edges.end()){
     if((*ite)->isSeam(gf)) return false;
@@ -1152,7 +1184,7 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
   if(all_vertices.size() == 3){
     MVertex *vv[3];
     int i = 0;
-    for(std::set<MVertex*>::iterator it = all_vertices.begin();
+    for(std::set<MVertex*, MVertexLessThanNum>::iterator it = all_vertices.begin();
 	it != all_vertices.end(); it++){
       vv[i++] = *it;
     }
@@ -1170,7 +1202,7 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
   std::vector<BDS_Point*> points(all_vertices.size());
   SBoundingBox3d bbox;
   int count = 0;
-  for(std::set<MVertex*>::iterator it = all_vertices.begin();
+  for(std::set<MVertex*, MVertexLessThanNum>::iterator it = all_vertices.begin();
       it != all_vertices.end(); it++){
     MVertex *here = *it;
     GEntity *ge = here->onWhat();
@@ -1266,7 +1298,7 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
   {
     std::vector<MVertex*> v;
     std::vector<MTriangle*> result;
-    v.insert(v.end(),all_vertices.begin(),all_vertices.end());
+    v.insert(v.end(), all_vertices.begin(), all_vertices.end());
 
     std::map<MVertex*,SPoint3> pos;
     for(unsigned int i = 0; i < v.size(); i++) {
@@ -1334,7 +1366,7 @@ bool meshGenerator(GFace *gf, int RECUR_ITER,
   }
 
 
-    // effectively recover the medge
+  // effectively recover the medge
   ite = edges.begin();
   while(ite != edges.end()){
     if(!(*ite)->isMeshDegenerated()){
@@ -2122,7 +2154,6 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
   {
     /// FIXME FOR PERIODIC : SOME MVERTices SHOULD BE DUPLICATED ...
     /// Still to be done...
-    printf("coucou1\n");
     std::vector<MVertex*> v;
     std::map<MVertex*, BDS_Point*> recoverMapInv;
     for(unsigned int i = 0; i < edgeLoops_BDS.size(); i++){
@@ -2134,7 +2165,7 @@ static bool meshGeneratorPeriodic(GFace *gf, bool debug = true)
       }
     }
 
-    printf("coucou2 %d verices\n",v.size());
+    //    printf("coucou2 %d verices\n",v.size());
     std::map<MVertex*,SPoint3> pos;
     for(unsigned int i = 0; i < v.size(); i++) {
       MVertex *v0 = v[i];
@@ -2815,6 +2846,64 @@ void partitionAndRemesh(GFaceCompound *gf)
 #endif
 }
 
+static bool getGFaceNormalFromVert(GFace *gf, MElement *el, SVector3 &nf)
+{
+  bool found = false;
+  for (int iElV = 0; iElV < el->getNumVertices(); iElV++) {
+    MVertex *v = el->getVertex(iElV);
+    SPoint2 param;
+    if(v->onWhat() == gf && v->getParameter(0, param[0]) &&
+       v->getParameter(1, param[1])) {
+      nf = gf->normal(param);
+      found = true;
+      break;
+    }
+  }
+  return found;
+}
+
+static bool getGFaceNormalFromBary(GFace *gf, MElement *el, SVector3 &nf)
+{
+  SPoint2 param(0., 0.);
+  bool ok = true;
+  for (int j = 0; j < el->getNumVertices(); j++) {
+    SPoint2 p;
+    // FIXME: use inexact reparam because some vertices might not be
+    // exactly on the surface after the 3D Delaunay
+    ok = reparamMeshVertexOnFace(el->getVertex(j), gf, p, false);
+    if (!ok) break;
+    param += p;
+  }
+  if (ok) {
+    param *= 1. / el->getNumVertices();
+    nf = gf->normal(param);
+  }
+  return ok;
+}
+
+static void getGFaceOrientation(GFace *gf, BoundaryLayerColumns *blc,
+                                bool existBL, bool fromVert,
+                                int &orientNonBL, int &orientBL)
+{
+  for (unsigned int iEl = 0; iEl < gf->getNumMeshElements(); iEl++) {
+    MElement *e = gf->getMeshElement(iEl);
+    const bool isBLEl = existBL &&
+                        (blc->_toFirst.find(e) != blc->_toFirst.end());
+    SVector3 nf;
+    if ((!isBLEl && orientNonBL == 0) || (isBLEl && orientBL == 0)) {       // Check only if orientation of BL/non-BL el. not already known
+      const bool found = fromVert ? getGFaceNormalFromVert(gf, e, nf) :
+                                    getGFaceNormalFromBary(gf, e, nf);
+      if (found) {
+        SVector3 ne = e->getFace(0).normal();
+        const int orient = (dot(ne, nf) > 0.) ? 1 : -1;
+        if (isBLEl) orientBL = orient;
+        else orientNonBL = orient;
+      }
+    }
+    if ((orientNonBL != 0) && (orientBL != 0)) break;                       // Stop when orientation found for non-BL and BL el.
+  }
+}
+
 void orientMeshGFace::operator()(GFace *gf)
 {
   if(!gf->getNumMeshElements()) return;
@@ -2827,8 +2916,8 @@ void orientMeshGFace::operator()(GFace *gf)
      gf->geomType() == GEntity::CompoundSurface){
     // don't do anything
   }
-  else{
-    // in old versions we checked the orientation by comparing the orientation
+  else {
+    // In old versions we checked the orientation by comparing the orientation
     // of a line element on the boundary w.r.t. its connected surface
     // element. This is probably better than what follows, but
     // * it failed when the 3D Delaunay changes the 1D mesh (since we don't
@@ -2836,66 +2925,48 @@ void orientMeshGFace::operator()(GFace *gf)
     // * it failed with OpenCASCADE geometries, where surface orientions do not
     //   seem to be consistent with the orientation of the bounding edges
 
-    bool done = false;
-    // first, try to find an element with one vertex categorized on the
-    // surface and for which we have valid surface parametric
-    // coordinates
-    for(unsigned int i = 0; i < gf->getNumMeshElements(); i++){
-      MElement *e = gf->getMeshElement(i);
-      for(int j = 0; j < e->getNumVertices(); j++){
-        MVertex *v = e->getVertex(j);
-        SPoint2 param;
-        if(v->onWhat() == gf && v->getParameter(0, param[0]) &&
-           v->getParameter(1, param[1])){
-          SVector3 nf = gf->normal(param);
-          SVector3 ne = e->getFace(0).normal();
-          if(dot(ne, nf) < 0){
-            Msg::Debug("Reversing orientation of mesh in face %d (param)", gf->tag());
-            for(unsigned int k = 0; k < gf->getNumMeshElements(); k++)
-              gf->getMeshElement(k)->reverse();
-          }
-          done = true;
-          break;
-        }
-      }
-      if(done) break;
-    }
+    // Now: orient surface elements w.r.t. normal to geometric model.
+    // Assumes that originally, orientation is consistent among boundary layer
+    // (BL) elements, and orientation is consistent among non-BL elements, but
+    // BL and non-BL elements can be oriented differently
 
-    if(!done){
-      // if we could not find such an element, just try to evaluate the
-      // normal at the barycenter of an element on the surface
-      for(unsigned int i = 0; i < gf->getNumMeshElements(); i++){
-        MElement *e = gf->getMeshElement(i);
-        SPoint2 param(0., 0.);
-        bool ok = true;
-        for(int j = 0; j < e->getNumVertices(); j++){
-          SPoint2 p;
-          // FIXME: use inexact reparam because some vertices might not be
-          // exactly on the surface after the 3D Delaunay
-          ok = reparamMeshVertexOnFace(e->getVertex(j), gf, p, false);
-          if(!ok) break;
-          param += p;
-        }
-        if(ok){
-          param *= 1. / e->getNumVertices();
-          SVector3 nf = gf->normal(param);
-          SVector3 ne = e->getFace(0).normal();
-          if(dot(ne, nf) < 0){
-            Msg::Debug("Reversing 2 orientation of mesh in face %d (cog)", gf->tag());
-            for(unsigned int k = 0; k < gf->getNumMeshElements(); k++)
-              gf->getMeshElement(k)->reverse();
-          }
-          done = true;
-          break;
-        }
-      }
-    }
+    // Determine whether there is a boundary layer (BL)
+    BoundaryLayerColumns *blc = gf->getColumns();
+    const bool existBL = !blc->_toFirst.empty();
 
-    if(!done)
+    // Get orientation of BL and non-BL elements.
+    // First, try to get normal to GFace from vertices.
+    // If it fails, try to get normal to GFace from element barycenter
+    int orientNonBL = 0, orientBL = existBL ? 0 : 1;
+    getGFaceOrientation(gf, blc, existBL, true, orientNonBL, orientBL);
+    if ((orientNonBL == 0) || (orientBL == 0))
+      getGFaceOrientation(gf, blc, existBL, false, orientNonBL, orientBL);
+
+    // Exit if could not determine orientation of both non-BL el. and BL el.
+    if ((orientNonBL == 0) && (orientBL == 0)) {
       Msg::Warning("Could not orient mesh in face %d", gf->tag());
+      return;
+    }
+
+    // Reverse BL and non-BL elements if needed
+    if (existBL) {                                                          // If there is a BL, test BL/non-BL elements
+      if ((orientNonBL == -1) || (orientBL == -1))
+        for (unsigned int iEl = 0; iEl < gf->getNumMeshElements(); iEl++) {
+          MElement *e = gf->getMeshElement(iEl);
+          if (blc->_toFirst.find(e) == blc->_toFirst.end()) {               // If el. outside of BL...
+            if (orientNonBL == -1) e->reverse();                            // ... reverse if needed
+          }
+          else                                                              // If el. in BL
+            if (orientBL == -1) e->reverse();                               // ... reverse if needed
+        }
+    }
+    else                                                                    // If no BL, reverse all elements if needed
+      if (orientNonBL == -1)
+        for (unsigned int iEl = 0; iEl < gf->getNumMeshElements(); iEl++)
+          gf->getMeshElement(iEl)->reverse();
   }
 
-  // apply user-specified mesh orientation constraints
+  // Apply user-specified mesh orientation constraints
   if(gf->meshAttributes.reverseMesh)
     for(unsigned int k = 0; k < gf->getNumMeshElements(); k++)
       gf->getMeshElement(k)->reverse();
