@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -22,8 +22,10 @@ class FunctionManager
   static FunctionManager *instance;
  public :
   static FunctionManager* Instance();
-  int createFunction(char *name, gmshFILE f, std::string &filename, int lineno);
-  int enterFunction(char *name, gmshFILE *f, std::string &filename, int &lineno) const;
+  int createFunction(const std::string &name, gmshFILE f,
+                     const std::string &filename, int lineno);
+  int enterFunction(const std::string &name, gmshFILE *f,
+                    std::string &filename, int &lineno) const;
   int leaveFunction(gmshFILE *f, std::string &filename, int &lineno);
 };
 
