@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2013 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -11,7 +11,7 @@
 #include "StringUtils.h"
 #include "GmshMessage.h"
 #include "GmshDefines.h"
-#include "MVertexPositionSet.h"
+#include "MVertexRTree.h"
 #include "Context.h"
 #include "adaptiveData.h"
 #include "OS.h"
@@ -392,6 +392,7 @@ bool PViewDataList::writePOS(const std::string &fileName, bool binary, bool pars
       int one = 1;
       if(!fwrite(&one, sizeof(int), 1, fp)){
         Msg::Error("Write error");
+        fclose(fp);
         return false;
       }
     }
@@ -450,9 +451,20 @@ static void createVertices(std::vector<double> &list, int nbelm, int nbnod,
   }
 }
 
+class nodeData{
+public:
+  int nbnod;
+  int nod;
+  double *data;
+  nodeData()
+    : nbnod(0), nod(0), data(0) {}
+  nodeData(int _nbnod, int _nod, double *_data)
+    : nbnod(_nbnod), nod(_nod), data(_data) {}
+};
+
 static void createElements(std::vector<double> &list, int nbelm, int nbnod,
-                           MVertexPositionSet &pos, std::vector<MElement*> &elements,
-                           double eps, int type)
+                           MVertexRTree &pos, std::vector<MElement*> &elements,
+                           int type, std::map<MVertex*, nodeData> *vertexData)
 {
   if(!nbelm) return;
   int t = 0;
@@ -519,16 +531,20 @@ static void createElements(std::vector<double> &list, int nbelm, int nbnod,
     double *y = &list[i + nbnod];
     double *z = &list[i + 2 * nbnod];
     std::vector<MVertex*> verts(nbnod);
-    for(int j = 0; j < nbnod; j++)
-      verts[j] = pos.find(x[j], y[j], z[j], eps);
+    for(int j = 0; j < nbnod; j++){
+      verts[j] = pos.find(x[j], y[j], z[j]);
+      if(vertexData)
+        (*vertexData)[verts[j]] = nodeData(nbnod, j, &list[i + 3 * nbnod]);
+    }
     MElement *e = factory.create(t, verts);
     elements.push_back(e);
   }
 }
 
 bool PViewDataList::writeMSH(const std::string &fileName, double version, bool binary,
-                             bool savemesh, bool multipleView,
-                             int partitionNum, bool saveInterpolationMatrices)
+                             bool saveMesh, bool multipleView,
+                             int partitionNum, bool saveInterpolationMatrices,
+                             bool forceNodeData)
 {
   if(_adaptive){
     Msg::Warning("Writing adapted dataset (will only export current time step)");
@@ -555,41 +571,51 @@ bool PViewDataList::writeMSH(const std::string &fileName, double version, bool b
     if(*numEle) numComponents = std::min(numComponents, numComp);
     createVertices(*list, *numEle, numNodes, vertices);
   }
-  MVertexPositionSet pos(vertices);
+  MVertexRTree pos(eps);
+  std::vector<MVertex*> unique;
+  for(unsigned int i = 0; i < vertices.size(); i++){
+    if(!pos.insert(vertices[i]))
+      unique.push_back(vertices[i]);
+  }
+  vertices.clear();
+
+  std::map<MVertex *, nodeData> vertexData;
 
   for(int i = 0; i < 24; i++){
     std::vector<double> *list = 0;
     int *numEle = 0, numComp, numNodes;
     int typ = _getRawData(i, &list, &numEle, &numComp, &numNodes);
-    createElements(*list, *numEle, numNodes, pos, elements, eps, typ);
+    createElements(*list, *numEle, numNodes, pos, elements, typ,
+                   forceNodeData ? &vertexData : 0);
   }
 
   int num = 0;
-  for(unsigned int i = 0; i < vertices.size(); i++)
-    if(vertices[i]->getIndex() < 0)
-      vertices[i]->setIndex(++num);
+  for(unsigned int i = 0; i < unique.size(); i++)
+    unique[i]->setIndex(++num);
 
   fprintf(fp, "$MeshFormat\n2.2 0 8\n$EndMeshFormat\n");
-  fprintf(fp, "$Nodes\n");
-  fprintf(fp, "%d\n", num);
-  for(unsigned int i = 0; i < vertices.size(); i++){
-    MVertex *v = vertices[i];
-    if(v->getIndex() > 0)
+
+  if(saveMesh){
+    fprintf(fp, "$Nodes\n");
+    fprintf(fp, "%d\n", (int)unique.size());
+    for(unsigned int i = 0; i < unique.size(); i++){
+      MVertex *v = unique[i];
       fprintf(fp, "%d %.16g %.16g %.16g\n", v->getIndex(), v->x(), v->y(), v->z());
-  }
-  fprintf(fp, "$EndNodes\n");
+    }
+    fprintf(fp, "$EndNodes\n");
 
-  fprintf(fp, "$Elements\n");
-  fprintf(fp, "%d\n", (int)elements.size());
-  for(unsigned int i = 0; i < elements.size(); i++){
-    if(version > 2.2)
-      Msg::Warning("Unable to write file in version '%g': using version 2.2",
-                   version);
-    elements[i]->writeMSH2(fp, 2.2, false, i + 1);
+    fprintf(fp, "$Elements\n");
+    fprintf(fp, "%d\n", (int)elements.size());
+    for(unsigned int i = 0; i < elements.size(); i++){
+      if(version > 2.2)
+        Msg::Warning("Unable to write file in version '%g': using version 2.2",
+                     version);
+      elements[i]->writeMSH2(fp, 2.2, false, i + 1);
+    }
+    fprintf(fp, "$EndElements\n");
   }
-  fprintf(fp, "$EndElements\n");
 
-  if(saveInterpolationMatrices && haveInterpolationMatrices()){
+  if(saveInterpolationMatrices && haveInterpolationMatrices() && !forceNodeData){
     fprintf(fp, "$InterpolationScheme\n");
     fprintf(fp, "\"INTERPOLATION_SCHEME\"\n");
     fprintf(fp, "%d\n", (int)_interpolation.size());
@@ -612,37 +638,59 @@ bool PViewDataList::writeMSH(const std::string &fileName, double version, bool b
   }
 
   for(int ts = 0; ts < NbTimeStep; ts++){
-    fprintf(fp, "$ElementNodeData\n");
-    if(saveInterpolationMatrices && haveInterpolationMatrices())
+    if(forceNodeData)
+      fprintf(fp, "$NodeData\n");
+    else
+      fprintf(fp, "$ElementNodeData\n");
+    if(saveInterpolationMatrices && haveInterpolationMatrices() && !forceNodeData)
       fprintf(fp, "2\n\"%s\"\n\"INTERPOLATION_SCHEME\"\n", getName().c_str());
     else
       fprintf(fp, "1\n\"%s\"\n", getName().c_str());
     fprintf(fp, "1\n%.16g\n", getTime(ts));
+    int size = forceNodeData ? (int)unique.size() : (int)elements.size();
     if(partitionNum)
-      fprintf(fp, "4\n%d\n%d\n%d\n%d\n", ts, numComponents, (int)elements.size(),
+      fprintf(fp, "4\n%d\n%d\n%d\n%d\n", ts, numComponents, size,
               partitionNum);
     else
-      fprintf(fp, "3\n%d\n%d\n%d\n", ts, numComponents, (int)elements.size());
-    num = 0;
-    for(int i = 0; i < 24; i++){
-      std::vector<double> *list = 0;
-      int *numEle = 0, numComp, numNodes;
-      int typ = _getRawData(i, &list, &numEle, &numComp, &numNodes);
-      if(*numEle){
-        int mult = numNodes;
-        if(_interpolation.count(typ))
-          mult = _interpolation[typ][0]->size1();
-        int nb = list->size() / *numEle;
-        for(unsigned int i = 0; i < list->size(); i += nb){
-          double *v = &(*list)[i + 3 * numNodes];
-          fprintf(fp, "%d %d", ++num, mult);
-          for(int j = 0; j < numComponents * mult; j++)
-            fprintf(fp, " %.16g", v[numComponents * mult * ts + j]);
-          fprintf(fp, "\n");
+      fprintf(fp, "3\n%d\n%d\n%d\n", ts, numComponents, size);
+
+    if(forceNodeData){
+      for(unsigned int i = 0; i < unique.size(); i++){
+        MVertex *v = unique[i];
+        fprintf(fp, "%d", v->getIndex());
+        int nbnod = vertexData[v].nbnod;
+        int nod = vertexData[v].nod;
+        double *d = vertexData[v].data;
+        for(int j = 0; j < numComponents; j++)
+          fprintf(fp, " %.16g",
+                  d[numComponents * nbnod * ts + numComponents * nod + j]);
+        fprintf(fp, "\n");
+      }
+      fprintf(fp, "$EndNodeData\n");
+    }
+    else{
+      int n = 0;
+      for(int i = 0; i < 24; i++){
+        std::vector<double> *list = 0;
+        int *numEle = 0, numComp, numNodes;
+        int typ = _getRawData(i, &list, &numEle, &numComp, &numNodes);
+        if(*numEle){
+          int mult = numNodes;
+          if(_interpolation.count(typ))
+            mult = _interpolation[typ][0]->size1();
+          int nb = list->size() / *numEle;
+          for(unsigned int i = 0; i < list->size(); i += nb){
+            double *v = &(*list)[i + 3 * numNodes];
+            fprintf(fp, "%d %d", ++n, mult);
+            for(int j = 0; j < numComponents * mult; j++)
+              fprintf(fp, " %.16g", v[numComponents * mult * ts + j]);
+            fprintf(fp, "\n");
+          }
         }
       }
+      fprintf(fp, "$EndElementNodeData\n");
     }
-    fprintf(fp, "$EndElementNodeData\n");
+
   }
 
   fclose(fp);
