@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -57,8 +57,8 @@ class PViewOptions {
     DoubleLogarithmic = 3
   };
 
-  int type;
-  int position[2], size[2], autoPosition;
+  int type, autoPosition;
+  double position[2], size[2];
   std::string format;
   int axes, axesAutoPosition, axesMikado;
   double axesTics[3];
