@@ -1,4 +1,4 @@
-// Gmsh - Copyright (C) 1997-2014 C. Geuzaine, J.-F. Remacle
+// Gmsh - Copyright (C) 1997-2015 C. Geuzaine, J.-F. Remacle
 //
 // See the LICENSE.txt file for license information. Please report all
 // bugs and problems to the public mailing list <gmsh@geuz.org>.
@@ -63,6 +63,18 @@ typedef struct _p_KSP* KSP;
 #define PetscOptionsGetBool PetscOptionsGetTruth
 #endif
 
+// The petsc3.5 change log says:
+// "KSPSetOperators() no longer has the MatStructure argument. The Mat objects now track that information themselves.
+// Use KPS/PCSetReusePreconditioner() to prevent the recomputation of the preconditioner if the operator changed
+// in the way that SAME_PRECONDITIONER did with KSPSetOperators()"
+// So I guess this should be called with PETSC_TRUE as second argument only for SAME_PRECONDITIONER
+// and false otherwise (i.e. for SAME_NONZERO_PATTERN, DIFFRENT_NONZERO_PATTERN) but it is a guess...
+#if (PETSC_VERSION_MAJOR < 3  || (PETSC_VERSION_MAJOR == 3 && PETSC_VERSION_MINOR < 5))
+#define KSPSetOperators(_ksp, _a, _b, SAME_PRECONDITIONER) KSPSetOperators(_ksp, _a, _b, SAME_PRECONDITIONER)
+#else
+# define SAME_PRECONDITIONER 999
+# define KSPSetOperators(_ksp, _a, _b, OPT_PRECON) (KSPSetOperators(_ksp, _a, _b), KSPSetReusePreconditioner(_ksp,PetscBool(OPT_PRECON == SAME_PRECONDITIONER)))
+#endif
 
 template <class scalar>
 class linearSystemPETSc : public linearSystem<scalar> {
@@ -102,7 +114,7 @@ class linearSystemPETSc : public linearSystem<scalar> {
   void zeroSolution();
   void printMatlab(const char *filename) const;
   virtual int systemSolve();
-  Mat &getMatrix(){ return _a; }
+  Mat getMatrix(){ return _a; }
   //std::vector<scalar> getData();
   //std::vector<int> getRowPointers();
   //std::vector<int> getColumnsIndices();
